# -*- coding:utf-8 -*-
# @time:2023/1/68:49
# @author: PyQt5学习爱好群 - 知行合一
# @file:comboCheckBox.py
# @software:PyCharm
from PyQtGuiLib.header import (
    QWidget,
    QComboBox,
    QLineEdit,
    QListView,
    QCompleter,
    QStyledItemDelegate,
    QLabel,
    QGraphicsDropShadowEffect,
    QStandardItemModel,
    QStandardItem,
    QMouseEvent,
    QColor,
    QFont,
    qt,
    Signal,
    QKeyEvent
)

from typing import Union
from PyQtGuiLib.core.comboBox.comboCheckBox.py_style import PyStyle
from PyQtGuiLib.core.comboBox.comboCheckBox.py_message_box import Py_Message_box


def show_text(function):
    def wrapped(self, *args, **kwargs):
        # 避免异常增加行////////////////////////////////////////////////////
        if self.vars["additem"]:
            if self.vars["listViewModel"].rowCount() - self.vars["rowcount"] == 1:
                self.vars["listViewModel"].removeRows(self.vars["rowcount"], 1)
                self.vars["lineEdit"].setFocus()
                Py_Message_box(parent=self._parent,
                               obj_name='mesBox',
                               title='提示',
                               width=200,
                               height=200,
                               message_txt='确保输入框光标在闪烁！否则无法取值。',
                               type='information')
                # 避免异常增加行、、、、、、、、、、、、、、、、、、、、、、、、、、、、、、、、、、、、、

        if self.vars["showTextLock"]:
            self.vars["showTextLock"] = False

            result = function(self, *args, **kwargs)
            items = self.get_selected()
            l = len(items)
            l_ = self.vars["listViewModel"].rowCount() - 1
            self.vars["listViewModel"].item(0).setCheckState(
                qt.Checked if l == l_ else qt.Unchecked if l == 0 else qt.PartiallyChecked)
            # 自定义 修改选中颜色
            for i in range(1, self.vars["listViewModel"].rowCount()):
                if str(self.vars["listViewModel"].item(i).checkState()) == 'CheckState.Checked':
                    self.vars["listViewModel"].item(i).setBackground(QColor(77, 42, 0, 30))
                else:
                    self.vars["listViewModel"].item(i).setBackground(QColor(39, 44, 54))
            # 自定义 修改选中颜色
            self.vars["lineEdit"].setText(
                "全选" if l == l_ else "无选择" if l == 0 else "||".join((item.text() for item in items)))
            if self.vars["lineEdit"].text() != '全选':
                self.setStyleSheet(PyStyle.ComBoStyle1)
            else:
                self.setStyleSheet(PyStyle.ComBoStyle)

            # 自定义 智能输入框
            # items_txt = self.get_items_txt()
            self.completer = QCompleter(self.get_items_txt())

            self.completer.setFilterMode(qt.MatchContains)
            self.completer.setCompletionMode(qt.PopupCompletion)
            self.vars["lineEdit"].setCompleter(self.completer)
            self.vars["lineEdit"].setFocus()

            dd=QStyledItemDelegate()
            self.completer.popup().setItemDelegate(dd)
            self.completer.popup().setStyleSheet(
            '''
            QAbstractItemView{
                background-color:rgba(44, 50, 61, 1);
                color:rgba(219, 226, 241, 1);
                border:1px solid rgba(37, 41, 48, 1);
                font: 15pt;
            }
            QAbstractItemView::item:hover{
                background-color:rgba(77, 155, 213, 1);
            }
            '''
            )
            self.completer.activated.connect(lambda x:self.only_text(x))
            self.vars["lineEdit"].setFocus()
            # 自定义 智能输入框
            self.vars["showTextLock"] = True
        else:
            result = function(self, *args, **kwargs)
        return result
    return wrapped


class _ToolTip(QLabel):
    # TOOLTIP / LABEL StyleSheet
    style_tooltip = """ 
    QLabel {{		
        background-color: {_dark_one};	
        color: {_text_foreground};
        padding-left: 10px;
        padding-right: 10px;
        border-radius: 17px;
        border: 0px solid transparent;
        border-right: 3px solid {_context_color};
        font: 800 9pt "Segoe UI";
    }}
    """
    def __init__(
        self,
        parent,
        tooltip,
        dark_one,
        context_color,
        text_foreground
    ):
        QLabel.__init__(self)

        # LABEL SETUP
        style = self.style_tooltip.format(
            _dark_one = dark_one,
            _context_color = context_color,
            _text_foreground = text_foreground
        )
        self.setObjectName(u"label_tooltip")
        self.setStyleSheet(style)
        self.setMinimumHeight(34)
        self.setParent(parent)
        self.setText(tooltip)
        self.adjustSize()

        # SET DROP SHADOW
        self.shadow = QGraphicsDropShadowEffect(self)
        self.shadow.setBlurRadius(30)
        self.shadow.setOffset(0,0)
        self.shadow.setColor(QColor(0, 0, 0, 80))
        self.setGraphicsEffect(self.shadow)


class ComboCheckBox(QComboBox):
    linSignal = Signal(object)

    class MyListView(QListView):
        def __init__(self, parent: QWidget = None, vars=None):
            super().__init__(parent)
            self.vars = vars

        def mousePressEvent(self, event: QMouseEvent):
            self.vars["lock"] = False
            super().mousePressEvent(event)

        def mouseDoubleClickEvent(self, event: QMouseEvent):
            self.vars["lock"] = False
            super().mouseDoubleClickEvent(event)

    def __init__(self,
                 parent: QWidget = None,
                 obj_name=None,
                 width:int=200,
                 height:int=35,
                 spacing:int=0,
                 punc:str=',',
                 head_word:str='',
                 end_word:str='',
                 is_edit:bool=True,
                 is_read_only:bool=True,
                 is_fill_bg:bool=False,
                 is_frame:bool=True,
                 style:str=None,
                 rowcount:int=0,
                 items=None
                 ):
        super().__init__(parent)
        self._parent = parent
        self._objName = obj_name
        self._width = width
        self._heigh = height
        self._spacing = spacing
        self._punc = punc
        self._headWord = head_word
        self._endWord = end_word
        self._isEdit = is_edit
        self._isReadOnly = is_read_only
        self._isFillBg = is_fill_bg
        self._isFrame = is_frame
        self._isShow = False
        self._style = style
        self._items=items

        self.vars = {
            "lock":True,
            "additem":False,
            "showTextLock":True,
            "lineEdit":QLineEdit(self),
            "listViewModel":QStandardItemModel(self),
            "rowcount":rowcount+1
        }
        self.vars["listView"] = self.MyListView(self, self.vars)
        self.vars["lineEdit"].setReadOnly(False)
        self.setModel(self.vars["listViewModel"])
        self.setView(self.vars["listView"])
        self.setLineEdit(self.vars["lineEdit"])

        self.add_item("全选")
        self.activated.connect(self.__show_selected)

        self.add_items(self._items)

        # 设置父窗口
        if self._parent is not None:
            self.setParent(self._parent)
        # 设置名称
        if self._objName is not None:
            self.setObjectName(self._objName)
        # 设置按钮的尺寸范围
        if self._width is None:
            if self._heigh is None:
                pass
            else:
                self.setMinimumHeight(self._heigh)
                self.setMaximumHeight(self._heigh)
        else:
            if self._heigh is None:
                self.setMinimumWidth(self._width)
                self.setMaximumWidth(self._width)
            else:
                self.setFixedSize(self._width, self._heigh)
        # 设置背景填充
        self.setAutoFillBackground(self._isFillBg)
        # 设置边框
        self.setFrame(self._isFrame)
        # 设置下拉框的显示对象，防止combo的设置影响qss样式
        # self.setView(QListView())
        # 设置显示框编辑状态
        if self._isEdit is not None:
            self.setEditable(self._isEdit)
        self.set_styleSheet(self._style)
        self.setMaxVisibleItems(30)

        self.__personalInformation = {
            "author": "PyQt5学习爱好群-知行合一",
            "contribution time": "2023.4.25"
        }

    def author(self) -> str:
        return self.__personalInformation

    def set_styleSheet(self, style:str=None):
        """设置样式"""
        if style is not None:
            self.setStyleSheet(style)
        else:
            self.setStyleSheet(PyStyle.ComBoStyle)

    def count(self) -> int:
        # 返回子项数
        return super().count() - 1

    @show_text
    def add_item(self, text,tooltip=None):
        #
        item = QStandardItem()
        item.setText(text)
        item.setCheckable(True)

        if tooltip is None:
            tooltip = text
        item.setToolTip(tooltip)
        font = QFont("黑体")
        item.setFont(font)
        item.setCheckState(qt.Checked)
        self.vars["listViewModel"].appendRow(item)

    @show_text
    def add_items(self, texts: Union[list,tuple]):
        self.vars["rowcount"] = len(texts) + 1
        # 根据文本列表添加子项
        for text in texts:
            self.add_item(text)
        self.vars["additem"] = True

    @show_text
    def add_item_data(self, name:str,crew_number:str,part_rate:str,namesake_rate:str,class_rate:str,code:str,system:str):
        item = QStandardItem()
        item.setText(name)
        item.setCheckable(True)

        # part_rate='60%'
        # chezuh='''{2010-{'06', '02', '03', '01', '07', '04', '00'}", "2023-{'01'}", "2009-{'06', '02', '03', '01', '07', '04', '00'}", "0208-{'06', '02', '03', '01', '07', '04', '00'}", "2007-{'06', '02', '03', '01', '07', '04', '00'}", "2181-{'03', '07'}", "2187-{'06', '02', '03', '01', '07', '04'}", "2008-{'06', '02', '04'}", "2030-{'01', '07', '04'}", "2143-{'04'}", "2012-{'06', '02', '03', '01', '07', '00'}", "2058-{'02'}", "2048-{'03', '06', '01'}", "2016-{'01'}", "2047-{'00'}", "2015-{'03', '07'}'''
        item.setToolTip("<h2>故障名称：<font color=#AAAAAA;size:10px>"+name+"</font></h2>"
                        "<h2>代码：<font color=#AAAAAA;size:18px>"+code+"</font> -----系统：<font color=#AAAAAA;size:18px>"+system+"</font></h2>"
                        "<h2>更换配件率：<font color=#AAAAAA;size:18px>"+part_rate+"</font></h2>"
                        "<h2>同名属实率：<font color=#AAAAAA;size:10px>"+namesake_rate+"</font></h2>"
                        "<h2>类名属实率：<font color=#AAAAAA;size:10px>"+class_rate+"</font></h2>"
                        "<h2>故障车组号：<font color=#AA0000;font-size:10px>"+crew_number+"</font></h2>")

        # item.setBackground(QColor(200, 200, 200))
        font = QFont("黑体")
        item.setFont(font)
        item.setCheckState(qt.Checked)
        self.vars["listViewModel"].appendRow(item)

    @show_text
    def add_items_data(self, data):
        self.vars["rowcount"] = len(data) + 1
        # 根据文本列表添加子项
        for row in data.itertuples():
            sanlv=row[3].split("||")
            part_rate=sanlv[0]   #换件
            namesake_rate=sanlv[1] #属实
            class_rate=sanlv[2] #类名
            code=row[1]
            name=row[2]
            crew_number=row[4]
            system=row[5]
            self.add_item_data(name,crew_number,part_rate,namesake_rate,class_rate,code,system)
        self.vars["additem"] = True

    @show_text
    def clear_items(self):
        # 清空所有子项
        self.vars["listViewModel"].clear()
        self.add_item("全选")

    def find_index(self, index: int):
        # 根据索引查找子项
        return self.vars["listViewModel"].item(index if index < 0 else index + 1)

    def find_indexs(self, indexs:Union[list,tuple]):
        # 根据索引列表查找子项
        return [self.find_index(index) for index in indexs]

    def find_text(self, text: str):
        # 根据文本查找子项
        tempList = self.vars["listViewModel"].findItems(text)
        tempList.pop(0) if tempList and tempList[0].row() == 0 else tempList
        return tempList

    def find_texts(self, texts: Union[list,tuple]):
        # 根据文本列表查找子项
        return {text: self.find_text(text) for text in texts}

    def in_text(self, text: str):
        # 根据文本查找子项
        for row in range(1, self.vars["listViewModel"].rowCount()):
            item = self.vars["listViewModel"].item(row)
            if text in item.text():
                # print(text,item.text())
                self.select_text(item.text(),True)
            else:
                self.select_del(item)

    def only_text(self, text: str):
        # 根据文本查找子项
        for row in range(1, self.vars["listViewModel"].rowCount()):
            item = self.vars["listViewModel"].item(row)
            if text in item.text():
                self.select_text(item.text(),True)
            else:
                self.select_del(item)
        self.linSignal.emit(self)

    @show_text
    def only_texts(self,textlist):
        for row in range(1, self.vars["listViewModel"].rowCount()):
            item = self.vars["listViewModel"].item(row)
            if item.text() in textlist:
                item.setCheckState(qt.Checked)
            else:item.setCheckState(qt.Unchecked)

    def get_text(self, index:int):
        # 根据索引返回文本
        return self.vars["listViewModel"].item(index if index < 0 else index + 1).text()

    def get_texts(self, indexs: Union[list,tuple]):
        # 根据索引列表返回文本
        return [self.get_text(index) for index in indexs]

    def change_text(self, index: int, text: str):
        # 根据索引改变某一子项的文本
        self.vars["listViewModel"].item(index if index < 0 else index + 1).setText(text)

    @show_text
    def select_index(self, index: int, state: bool = True):
        # 根据索引选中子项，state=False时为取消选中
        self.vars["listViewModel"].item(index if index < 0 else index + 1).setCheckState(
            qt.Checked if state else qt.Unchecked)

    @show_text
    def select_indexs(self, indexs: Union[list,tuple], state: bool = True):
        # 根据索引列表选中子项，state=False时为取消选中
        for index in indexs:
            self.select_index(index, state)

    @show_text
    def select_text(self, text: str, state: bool = True):
        # 根据文本选中子项，state=False时为取消选中
        for item in self.find_text(text):
            item.setCheckState(qt.Checked if state else qt.Unchecked)

    @show_text
    def select_texts(self, texts: Union[list,tuple], state: bool = True):
        # 根据文本列表选中子项，state=False时为取消选中
        for text in texts:
            self.select_text(text, state)

    @show_text
    def select_reverse(self):
        # 反转选择
        if self.vars["listViewModel"].item(0).checkState() == qt.Unchecked:
            self.select_all()
        elif self.vars["listViewModel"].item(0).checkState() == qt.Checked:
            self.select_clear()
        else:
            for row in range(1, self.vars["listViewModel"].rowCount()):
                self.__select_reverse(row)

    def __select_reverse(self, row: int):
        item = self.vars["listViewModel"].item(row)
        item.setCheckState(qt.Unchecked if item.checkState() == qt.Checked else qt.Checked)

    @show_text
    def deselected(self, text):
        #去除选择单个
        if text:
            ele_list = self.find_text(text)
            for ele in ele_list:
              ele.setCheckState(qt.Unchecked)

    @show_text
    def select_all(self):
        # 全选
        for row in range(0, self.vars["listViewModel"].rowCount()):
            self.vars["listViewModel"].item(row).setCheckState(qt.Checked)

    @show_text
    def select_clear(self):
        # 全不选
        for row in range(0, self.vars["listViewModel"].rowCount()):
            self.vars["listViewModel"].item(row).setCheckState(qt.Unchecked)
    @show_text
    def select_del(self,item:object):
        # 不选
        item.setCheckState(qt.Unchecked)

    @show_text
    def remove_index(self, index: int):
        # 根据索引移除子项
        return self.vars["listViewModel"].takeRow(index if index < 0 else index + 1)

    @show_text
    def remove_indexs(self, indexs:Union[list,tuple]):
        # 根据索引列表移除子项
        return [self.remove_index(index) for index in sorted(indexs, reverse=True)]

    @show_text
    def remove_text(self, text: str):
        # 根据文本移除子项
        items = self.find_text(text)
        indexs = [item.row() for item in items]
        return [self.vars["listViewModel"].takeRow(index) for index in sorted(indexs, reverse=True)]

    @show_text
    def remove_texts(self, texts:Union[list,tuple]):
        # 根据文本列表移除子项
        return {text: self.remove_text(text) for text in texts}

    def get_selected(self):
        # 获取当前选择的子项
        items = list()
        for row in range(1, self.vars["listViewModel"].rowCount()):
            item = self.vars["listViewModel"].item(row)
            if item.checkState() == qt.Checked:
                items.append(item)
        return items

    def get_selected_txt(self):
        # 获取当前选择的子项
        items = list()
        for row in range(1, self.vars["listViewModel"].rowCount()):
            item = self.vars["listViewModel"].item(row)
            if item.checkState() == qt.Checked:
                items.append(item.text())
        return items

    def get_items_txt(self):
        # 获取当前选择的子项
        items = list()
        for row in range(1, self.vars["listViewModel"].rowCount()):
            item = self.vars["listViewModel"].item(row)
            items.append(item.text())
        return items

    def get_items(self):
        # 获取当前选择的子项
        items = list()
        for row in range(1, self.vars["listViewModel"].rowCount()):
            item = self.vars["listViewModel"].item(row)
            items.append(item)
        return items

    def is_all(self):
        # 判断是否是全选
        return True if self.vars["listViewModel"].item(0).checkState() == qt.Checked else False

    def sort(self, order=qt.AscendingOrder):
        # 排序，默认正序
        self.vars["listViewModel"].sort(0, order)

    @show_text
    def __show_selected(self, index):
        if not self.vars["lock"]:
            if index == 0:
                if self.vars["listViewModel"].item(0).checkState() == qt.Checked:
                    self.select_clear()
                else:
                    self.select_all()
            else:
                self.__select_reverse(index)

            self.vars["lock"] = True

    def hidePopup(self):
        if self.vars["lock"]:
            self.linSignal.emit(self)
            super().hidePopup()

    def keyPressEvent(self, event:QKeyEvent):
        if (event.key() != qt.Key_Enter and event.key() != qt.Key_Return) and (event.key() != qt.Key_Backspace):
            txt=self.vars["lineEdit"].text()
            n=self.vars["lineEdit"].cursorPosition()
            etxt=event.text()
            list_i = list(txt)  # str -> list
            list_i.insert(n, etxt)  # 注意不用重新赋值
            str_i = ''.join(list_i)  # list -> str
            self.vars["lineEdit"].setText(str_i)
        elif (event.key() == qt.Key_Enter) or event.key() == qt.Key_Return:
            if self.vars["lineEdit"].text() == '全选' or self.vars["lineEdit"].text().strip() == '':
                self.select_all()
            else:
                self.in_text(self.vars["lineEdit"].text())
        elif (event.key() == qt.Key_Backspace):
            if self.vars["lineEdit"].text()=='全选' or len(self.vars["lineEdit"].text())>10 or self.vars["lineEdit"].text()=='无选择':
                self.setEditText('')
            super().keyPressEvent(event)



# -*- coding:utf-8 -*-
# @time:2023/4/1316:32
# @author:LX
# @file:test_CircularProgressBar.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QMainWindow,
    QLabel,
    QColor,
    QFont,
    QVBoxLayout,
    QHBoxLayout,
    QWidget,
    QTimer,
    QPushButton
)

'''
    CircularProgressBar 测试用例
'''

from PyQtGuiLib.core.progressBar import CircularProgressBar


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.resize(800, 600)

        self.progress1 = CircularProgressBar(self)
        self.progress2 = CircularProgressBar(self)
        self.progress3 = CircularProgressBar(self)

        label1 = QLabel("默认风格:")
        label2 = QLabel("自定义风格:")
        label3 = QLabel("动画:", self)

        self.progress2.setBgColor("#444444")
        self.progress2.setProgressColor("#ff00ff")
        self.progress2.setTextColor("red")
        self.progress2.setFormat("{:.1f} %")
        self.progress2.setFont(self.font())

        self.progress3.setBgColor(QColor(0, 0, 0, 50))
        self.progress3.setProgressColor(QColor(255, 102, 153, 255))
        self.progress3.setFont(QFont("HarmonyOS Sans SC", 20))
        self.progress3.setChunkWidth(20)
        self.progress3.setRange(300, 500)
        self.progress3.setStyleSheet("background-color:transparent")

        label3.move(150, 60)
        self.progress3.setGeometry(100, 100, 150, 150)

        # 设置进度
        self.progress1.setValue(50)
        self.progress2.setValue(75)
        # self.progress3.animate(100, 3000)

        vbox = QVBoxLayout()
        vbox.addWidget(label1)
        vbox.addWidget(self.progress1)
        vbox.addWidget(label2)
        vbox.addWidget(self.progress2)

        hbox = QHBoxLayout()
        hbox.addStretch(1)
        hbox.addLayout(vbox)
        hbox.addStretch(1)

        centralWidget = QWidget()
        centralWidget.setLayout(hbox)
        self.setCentralWidget(centralWidget)

        self._startValue = 300
        self._timer = QTimer(self)
        self._timer.setInterval(200)
        self._timer.timeout.connect(self.setProssValue)

        self.btn = QPushButton("开始", self)
        self.btn.move(125, 300)
        self.btn.clicked.connect(self._timer.start)

    def setProssValue(self):
        self.progress3.setValue(self._startValue)
        self._startValue += 1



if __name__ == '__main__':
    app = QApplication(sys.argv)

    win = MainWindow()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# self implementation generated from reading ui file 'test.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


import sys
from PyQt5.QtWidgets import QApplication, QWidget
from PyQt5 import QtCore, QtWidgets

from PyQtGuiLib.styles.qssFile.drak.qssDrak import QSSDrak
from PyQtGuiLib.styles import QssStyleAnalysis


class Test(QWidget):
    def __init__(self, *args,**kwargs) -> None:
        super().__init__(*args,**kwargs)
        self.setupUi()

        # QSS解析库
        self.qss = QssStyleAnalysis(self)
        self.qss.setQSS(QSSDrak)
    
    def setupUi(self):
        self.setObjectName("self")
        self.resize(1042, 699)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.widget = QtWidgets.QWidget(self)
        self.widget.setObjectName("widget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lineEdit = QtWidgets.QLineEdit(self.widget)
        self.lineEdit.setMaximumSize(QtCore.QSize(16777215, 31))
        self.lineEdit.setObjectName("lineEdit")
        self.horizontalLayout.addWidget(self.lineEdit)
        self.pushButton = QtWidgets.QPushButton(self.widget)
        self.pushButton.setMinimumSize(QtCore.QSize(0, 41))
        self.pushButton.setObjectName("pushButton")
        self.horizontalLayout.addWidget(self.pushButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.listWidget = QtWidgets.QListWidget(self.widget)
        self.listWidget.setObjectName("listWidget")
        self.verticalLayout.addWidget(self.listWidget)
        self.horizontalLayout_2.addWidget(self.widget)
        self.tabWidget = QtWidgets.QTabWidget(self)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.tableWidget = QtWidgets.QTableWidget(self.tab)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setColumnCount(5)
        self.tableWidget.setRowCount(7)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(4, item)
        self.verticalLayout_2.addWidget(self.tableWidget)
        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.tabWidget.addTab(self.tab_2, "")
        self.horizontalLayout_2.addWidget(self.tabWidget)

        self.retranslateUi()
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(self)

    def retranslateUi(self):
        _translate = QtCore.QCoreApplication.translate
        self.setWindowTitle(_translate("self", "self"))
        self.pushButton.setText(_translate("self", "查找"))
        item = self.tableWidget.verticalHeaderItem(0)
        item.setText(_translate("self", "新建行"))
        item = self.tableWidget.verticalHeaderItem(1)
        item.setText(_translate("self", "新建行"))
        item = self.tableWidget.verticalHeaderItem(2)
        item.setText(_translate("self", "新建行"))
        item = self.tableWidget.verticalHeaderItem(3)
        item.setText(_translate("self", "新建行"))
        item = self.tableWidget.verticalHeaderItem(4)
        item.setText(_translate("self", "新建行"))
        item = self.tableWidget.verticalHeaderItem(5)
        item.setText(_translate("self", "新建行"))
        item = self.tableWidget.verticalHeaderItem(6)
        item.setText(_translate("self", "新建行"))
        item = self.tableWidget.horizontalHeaderItem(0)
        item.setText(_translate("self", "新建列"))
        item = self.tableWidget.horizontalHeaderItem(1)
        item.setText(_translate("self", "新建列"))
        item = self.tableWidget.horizontalHeaderItem(2)
        item.setText(_translate("self", "新建列"))
        item = self.tableWidget.horizontalHeaderItem(3)
        item.setText(_translate("self", "新建列"))
        item = self.tableWidget.horizontalHeaderItem(4)
        item.setText(_translate("self", "新建列"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("self", "Tab 1"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("self", "Tab 2"))

if __name__ == '__main__':
    app = QApplication(sys.argv)

    win = Test()
    win.show()

    sys.exit(app.exec_())
    


# -*- coding:utf-8 -*-
# @time:2023/3/2011:39
# @author:LX
# @file:animationLayout.py
# @software:PyCharm

from PyQtGuiLib.header import (
    QPaintEvent,
    QRect
)

'''
    
    动画上的布局
'''


class AnimationLayout:
    def __init__(self,pe:QPaintEvent):
        self.__pe = pe

        #
        self.__minHeight = 23

        self.__space = 6

        self.__topMargin = 9
        self.__bottomMargin = 9
        self.__leftMargin = 9
        self.__rightMargin = 9

        self.__draw_list = []
        print(self.rect())

    def rect(self) -> QRect:
        return self.__pe.rect()

    def minHeight(self) -> int:
        return self.__minHeight

    def setMinHeight(self,h:int):
        self.__minHeight = h

    def space(self) -> int:
        return self.__space

    def setSpace(self,v:int):
        self.__space = v

    def setContentsMargins(self,top,bottom,left,right):
        self.__topMargin = top
        self.__bottomMargin = bottom
        self.__leftMargin = left
        self.__rightMargin = right

    def contentsMargins(self)->tuple:
        return self.__topMargin,self.__bottomMargin,self.__leftMargin,self.__rightMargin

    # 子类重写函数
    def __addDraw__(self,graph,*args):
        return

    # 调用函数
    def addDraw(self,graph,*args):
        self.__draw_list(self.__addDraw__(graph,*args))

    def count(self) -> int:
        return self.__draw_list

    def isEmpty(self) -> bool:
        return False if self.__draw_list else True

    def drawAt(self,i:int):
        return self.__draw_list[i]


# -*- coding:utf-8 -*-
# @time:2023/2/189:55
# @author:LX
# @file:styleLinker.py
# @software:PyCharm

'''
    样式链接器(属于外置工具)
    解决在程序运行过中,去动态的调节样式

'''
import re
import uuid
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QPushButton,
    QLabel,
    QObject,
    QTreeWidgetItem,
    QLineEdit,
    Qt,
    QMenu,
    QAction,
    QCursor,
    QListWidget,
    QListWidgetItem,
    QMessageBox
)
from PyQtGuiLib.styles import QssStyleAnalysis
from functools import partial
from PyQtGuiLib.core.flowLayout import FlowLayout
from abandonCase.linker.styleLinkerUi import StyleLinkerUI

from abandonCase.linker.controlType import getStyleLists,getStyleCommentLists,getMergeStyles
from abandonCase.linker.component import (
    RegisterComponent
)


# 改类是TAB上创建的QWidget
class TWidget(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.__flow = FlowLayout(self)
        self.__Hand_Reg_Funs = []

        # 保存所有创建的菜单项
        self._acs = []
        # 记录已经激活的控件
        self.activate_control = []

    def setObj(self,parent):
        self._parent = parent

    def setHandRegFuns(self,gs):
        '''
            保存 需要手动添加的组件,
            这些设置 完成后, 注册右键菜单,可以通过右键菜单来添加小组件
        :param gs:
        :return:
        '''
        self.__Hand_Reg_Funs = gs
        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.menu_event)

    # 添加一个 小组件窗口,并自动布局
    def addGroupBox(self,widget):
        self.__flow.addWidget(widget)

    def isActivate(self,obj)->bool:
        if obj.title() in self.activate_control:
            QMessageBox.warning(self,"警告","{}该组件已激活".format(obj.title()))
            return True
        return False

    def menu_event(self):
        self.menu = QMenu(self)

        def p_(obj):
            if self.isActivate(obj) is False:
                obj.module()
                self.activate_control.append(obj.title())

        for regClass in self.__Hand_Reg_Funs:
            reg = regClass(self._parent,self)
            ac = QAction(reg.title())
            self._acs.append(ac)
            self.menu.addAction(ac)
            ac.triggered.connect(partial(p_,reg))


        self.menu.popup(QCursor.pos())

'''
    动态样式链接器
'''

class StyleLinker(StyleLinkerUI):
    def __init__(self):
        super().__init__()

        # 控件对象列表, qss解析对象列表,全局共享QSS解析器操作对象,全局共享选择器(在切换tab时切换)
        '''
            self.global_var 在切换根节点的同时,设置 QSS解析器对象
            self.global_select 在切换Tab的同时,切换当前 QSS解析器对象 的 选择器对象
        '''
        self.__objs = []
        self.__qssAny = []  # type:[QssStyleAnalysis]
        self.global_var = None  # type: QssStyleAnalysis
        self.global_select = None # type:str

        # 记录根节点对象,同时记录QSS解析器对象,方便后期判断
        # [{"root":Xxx,"qss":xxx}]
        self.record_tree_root = []
        self.buffer_tree_root = [] # 缓冲区

        self.myEvent()

    def addQObject(self,obj:QObject):
        '''
            在添加控件对象的同时,创建该对象QSS解析器对象
            如果该控件有样式,则继承,如果没有则创建出一个默认的空样式,
            还会创建一个对应的树节点(父节点+子节点),
                        --- 后期,根据点击树节点,创建一系列tab节点
        '''
        if obj in self.__objs:  # 如果出现重复对象,直接忽略
            return

        qssaAng = QssStyleAnalysis(obj)
        bool_style = bool(obj.styleSheet())
        q = re.findall("QtWidgets\.(.*)\'|__main__\.(.*)\'", str(type(obj)))[0]
        if q:
            q = q[0] if q[0] else q[1]
        else:
            raise Exception("Match failure!")
        qss = '''
        %s{

        }
        ''' % q
        if bool_style:
            qssaAng.setQSS(obj.styleSheet())
            qssaAng.appendQSS(qss)
        else:
            qssaAng.setQSS(qss)

        '''
            名称标记 获取名称读顺序
            优先读取 text() 方法,如果没有该方法,或者是该方法返回的数据为空时,
            读取objectName()方法,如果为空,
            则读取这个对象实例化的十六进制,但正则匹配出现错误时,
            直接生成uuid作为名称
        '''
        name_flag = False
        # 获取名称
        if hasattr(obj, "text"):
            if obj.text():
                name = obj.text()
                name_flag = True
        if name_flag is False and obj.objectName():
            name = obj.objectName()
            name_flag = True
        if name_flag is False:
            try:
                name = re.findall("(0x.*)>", str(obj))[0]
            except Exception as e:
                name = uuid.uuid4()
        # 创建树根节点,以及子节点
        treeRoot = QTreeWidgetItem(self.tree())
        treeRoot.setText(0,name)
        self.record_tree_root.append({"root":treeRoot,"qss":qssaAng})

        '''
            对样式进行融合,如果该样式没有在QSS解析器中,则添加,
            最后在创建出子节点,再创建子节点的同时补上样式说明
        '''
        child_notes = getMergeStyles(qssaAng.header(),getStyleLists(q))
        for select in child_notes:
            if qssaAng.isSelectKey(select) is False:
                qssaAng.appendQSSDict({
                    select:{}
                })
        # 获取样式的说明
        comments = getStyleCommentLists(q)
        child_notes_zip = zip(child_notes,comments)
        for select,comment in child_notes_zip:
            child_tree = QTreeWidgetItem(treeRoot)
            child_tree.setText(0,select)
            child_tree.setToolTip(0,comment)

        self.__qssAny.append(qssaAng)
        self.__objs.append(obj)

    def addQObjects(self,objs):
        for obj in objs:
            self.addQObject(obj)

    # ----------------------

    def changeTab_Event(self,index):
        '''
            在切换 Tab 的同时根据tab的名称,区切换,当前QSS解析器对象的选择器名称
        '''
        if index:
            tab_name = self.tab().tabText(index)
            self.global_select = tab_name

    def doubleClickTree_Event(self,item:QTreeWidgetItem):
        '''
            双击树,获取根节点,同时设置为全局共享的操作对象,每当获取一个根节点,
            就将这个根节点加入缓冲区,方便下次查找,
            在通过根节点,找到下面的子节点,并创建出tab,
            在切换根节点时,先清空tab再创建,

            如果点击的是子节点,
                如果table存在,则切换换到对应table,如果不存在,则什么都不做
                这一段是存在BUG的(2023.2.20)
        '''

        # 判断是否点击的子节点
        child_flag = True
        for d in self.record_tree_root:
            if item == d["root"]:
                child_flag = False
                break

        if child_flag:
            child_name = item.text(0)
            table_count = self.tab().count()
            for i in range(table_count):
                if self.tab().tabText(i) == child_name:
                    self.tab().setCurrentIndex(i)
                    self.global_select = child_name # 重新设置选择器
                    break
            return
        # ------

        tab_count = self.tab().count()
        if tab_count > 1:
            for i in range(tab_count, 0, -1):
                self.tab().removeTab(i)

        root = None  # type:QTreeWidgetItem

        if item in self.buffer_tree_root: # 在缓冲区里面查找
            root = item
            for d in self.buffer_tree_root:
                if d["root"] == item:
                    self.global_var = d["qss"]
                    self.showStyleBrowserCode(self.global_var.toStr())
                    break

        if root is None:
            for d in self.record_tree_root:
                if d["root"] == item:
                    root = item
                    self.global_var = d["qss"]
                    self.buffer_tree_root.append(d)
                    self.showStyleBrowserCode(self.global_var.toStr())
                    break

        if root:
            for c in range(root.childCount()):
                child = root.child(c)
                tab_name = child.text(0)
                self.createTab(tab_name)

    # 通过树的根节点来查找 QSS解析器对象
    def getObj(self,root:QTreeWidgetItem) -> QssStyleAnalysis:
        for d in self.record_tree_root:
            if d["root"] == root:
                return d["qss"]

    def showStyleBrowserCode(self,style):
        self.browser().clear()
        self.browser().append(style)

    # 注册组件
    def registerComponent(self,wid):
        for regclass in RegisterComponent.getRegister():
            regclass(self,wid).module()

        wid.setObj(self)
        wid.setHandRegFuns(RegisterComponent.getHandRegister())

    # 创建tab
    def createTab(self, name):
        wid = TWidget()
        # flow = FlowLayout(wid) # 加入流式布局
        # ----
        ''''
            在wid上面添加各种组件
        '''
        self.registerComponent(wid)
        # ----
        self.tab().addTab(wid,name)

    def myEvent(self):
        self.tab().tabBarClicked.connect(self.changeTab_Event)
        self.tree().itemDoubleClicked.connect(self.doubleClickTree_Event)

# =====================================
# 下面是测试用例

class Test(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,600)

        self.btn = QPushButton("测试1",self)
        self.btn.setStyleSheet('''
        QPushButton{
        background-color: rgb(255, 85, 127);
        }
        QPushButton:hover{
        background-color: rgb(0, 85, 127);
        }
        ''')
        self.l2 = QLabel("我是标签",self)
        self.btn.move(10,10)
        self.l2.move(40,40)
        self.line = QLineEdit(self)
        self.line.move(10,80)

        # listwidget
        self.listw = QListWidget(self)
        self.listw.setObjectName("listw")
        self.listw.move(200,30)
        self.listw.resize(130,230)
        item1 = QListWidgetItem("test1")
        item2 = QListWidgetItem("test2")
        self.listw.addItem(item1)
        self.listw.addItem(item2)


        self.styleLinker = StyleLinker()
        self.styleLinker.addQObjects([self.btn,self.l2,self.line,self.listw])
        self.styleLinker.show()

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = Test()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2023/4/717:20
# @author:LX
# @file:eg2.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QColor,
    Qt,
    qt
)
from PyQtGuiLib.styles import SuperPainter



class Test(QWidget):
    def __init__(self):
        super().__init__()
        '''
            SuperPainter第二个案例,私有属性
            在上一个案例中会发现在画绿色和蓝色的矩形的时候,每次都需要去设置画笔,
            接下来演示 私有属性 来做这件事件
        '''
        self.painter = SuperPainter() # 创建一个画师对象,注意这个对象不要创建在paintEvent里面

    def paintEvent(self, QPaintEvent):
        self.painter.begin(self)

        # 首先画一个没有任何样式的矩形
        self.painter.drawRect(20,20,50,50)

        # 设置一个画笔,在画两个矩形,这里我们可以看到两个红色边框的画笔
        self.painter.setPen(QColor("red"))
        self.painter.drawRect(100, 20, 50, 50)
        self.painter.drawRect(200, 20, 50, 50)

        '''
            接下来我们画一个,绿色和蓝色的矩形,使用私有属性,
            只需要通过openAttr来设置
        '''
        self.painter.drawRect(20, 80, 50, 50,openAttr={"color":"green"})
        self.painter.drawRect(100, 80, 50, 50,openAttr={"color":"blue"})
        '''
            看到这里,你会发现,这不就是换了写法吗,
            那么再画一个图形,你会发现这个矩形的边框是红色(上面有设置画笔),并不是blue,
            这就是私有属性的好处,私有有属性不会影响下面其他图形的颜色,
            如果是通过画笔设置的话,是会影响下面画的图形的颜色
        '''
        self.painter.drawRect(200,80,50,50)

        self.painter.end()

if __name__ == '__main__':
    app = QApplication(sys.argv)

    win = Test()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



# -*- coding:utf-8 -*-
# @time:2023/5/299:21
# @author:LX
# @file:area.py
# @software:PyCharm
from PyQtGuiLib.header import (
    QApplication,
    sys,
    QWidget,
    QVBoxLayout,
    QHBoxLayout,
    QPushButton,
    Signal,
    QLabel,
    QLineEdit,
    QPainter,
    QSpacerItem,
    QSizePolicy,
    Qt,
    QColor,
    QPixmap,
    qt,
    QGradient,
    QLinearGradient,
    QRadialGradient,
    QConicalGradient,
    QResizeEvent,
    QSize,
    QWindowStateChangeEvent,
    QPoint,
    QMenu,
    QAction,
    QCursor,
    QColorDialog,
    QFrame
)
import math

Handle_pure = "pure"
Handle_Linear = "linear"
Handle_Radial = "radial"
Handle_Conical = "conical"

G_Mode_Pad = "pad"
G_Mode_Repeat = "repeat"
G_Mode_Reflect = "reflect"

from PyQtGuiLib.styles.superPainter.superPainter import SuperPainter, VirtualObject


class ColorHsv(QWidget):
    rgbaChange = Signal(QColor)

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(300, 50)

        self.suppainter = SuperPainter()

        self.percentage_pos = 0.0

        self.updateHSV()

    def createPix(self):
        if not hasattr(self,"pix"):
            self.pix = QPixmap(self.size())
        else:
            self.pix = self.pix.scaled(self.size())
        self.pix.fill(qt.transparent)

    def drawHSV(self):
        painter = QPainter(self.pix)
        painter.setRenderHints(qt.Antialiasing)
        gradient = QLinearGradient(self.width(), 0, 0, 0)

        i = 0.0
        gradient.setColorAt(0, QColor.fromHsvF(0, 1, 1, 1))
        while i < 1.0:
            gradient.setColorAt(i, QColor.fromHsvF(i, 1, 1, 1))
            i += 1.0 / 16.0
        gradient.setColorAt(1, QColor.fromHsvF(0, 1, 1, 1))
        painter.setPen(qt.NoPen)
        painter.setBrush(gradient)
        painter.drawRoundedRect(self.rect(), 0, 0)

    def updateHSV(self):
        self.createPix()
        self.drawHSV()

    # The percentage of the updated location
    def updatePercentagePos(self,cur_x:int):
        self.percentage_pos = round(cur_x / self.width(), 2)

    def sendEMitColor(self,x,y):
        self.updatePercentagePos(x)
        self.rgbaChange.emit(self.pix.toImage().pixelColor(x, y))

    def getColor(self)->QColor:
        cursor = self.suppainter.virtualObj("cursor")
        return self.pix.toImage().pixelColor(*cursor.getPos())

    def mouseMoveEvent(self, e) -> None:
        x = e.pos().x()
        cursor = self.suppainter.virtualObj("cursor")
        y,w = cursor.getVirtualArgs()[1:3]
        if 0 <= x and x <= self.width() - w:
            cursor.move(x, y)
            self.update()
            self.sendEMitColor(x,y)
        super().mouseMoveEvent(e)

    def mousePressEvent(self, e) -> None:
        cursor = self.suppainter.virtualObj("cursor")
        y = cursor.getVirtualArgs()[1]
        x = e.pos().x()
        cursor.move(x, y)
        cursor.updateIndexToArgs(2, 4)
        self.update()
        self.sendEMitColor(x,y)
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e) -> None:
        cursor = self.suppainter.virtualObj("cursor")
        cursor.updateIndexToArgs(2, 1)
        self.update()
        super().mouseReleaseEvent(e)

    def paintEvent(self, e) -> None:
        self.suppainter.begin(self)
        self.suppainter.drawPixmap(e.rect(), self.pix)
        self.suppainter.drawRect(0, 0, 1, self.height(), openAttr={"color": "white", "width": 1},
                                 virtualObjectName="cursor")
        self.suppainter.end()

    def resizeEvent(self, e:QResizeEvent) -> None:
        if self.suppainter.isVirtualObj("cursor"):
            cursor = self.suppainter.virtualObj("cursor")
            x = int(self.percentage_pos*e.size().width())
            cursor.updateIndexToArgs(0,x)
        self.updateHSV()
        super().resizeEvent(e)

    def changeEvent(self, e:QWindowStateChangeEvent) -> None:
        if e.type() == e.WindowStateChange:
            if self.suppainter.isVirtualObj("cursor") and (self.isMaximized() or self.isMinimized()):
                cursor = self.suppainter.virtualObj("cursor")
                cursor.updateIndexToArgs(0,self.percentage_pos*self.width())
                self.updateHSV()
        super().changeEvent(e)


class ColorBar(QWidget):
    rgbaChange = Signal(QColor)

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.suppainter = SuperPainter()

        self.__bgColor = QColor(0, 255, 0, 255)

        self.is_hide_hand = False

        self.percentage_pos = {
            "x":0.0,
            "y":0.0
        }

        self.updateLinear()

    def setHideHand(self,b:bool):
        self.is_hide_hand = b

    def updateBgColor(self,color:QColor):
        self.__bgColor = color
        self.updateLinear()
        self.update()

    def createPix(self):
        if not hasattr(self, "pix"):
            self.pix = QPixmap(self.size())
        else:
            self.pix = self.pix.scaled(self.size())

        if not hasattr(self, "gray_pix"):
            self.gray_pix = QPixmap(self.size())
        else:
            self.gray_pix = self.gray_pix.scaled(self.size())

        self.gray_pix.fill(qt.transparent)
        self.pix.fill(qt.transparent)

    def drawLinear(self):
        painter = QPainter(self.pix)
        painter.setRenderHints(qt.Antialiasing)

        gradient = QLinearGradient(0, 0, self.width(), 0)
        gradient.setColorAt(0, QColor("#fff"))
        gradient.setColorAt(1, self.__bgColor)

        painter.setPen(qt.NoPen)
        painter.setBrush(gradient)
        painter.drawRoundedRect(self.rect(), 2, 2)

        painter_gray = QPainter(self.gray_pix)
        painter_gray.setRenderHints(qt.Antialiasing)

        gradient = QLinearGradient(0, 0, 0, self.height())
        gradient.setColorAt(0, QColor(0, 0, 0, 0))
        gradient.setColorAt(1, QColor("#000"))
        painter_gray.setPen(qt.NoPen)
        painter_gray.setBrush(gradient)
        painter_gray.drawRoundedRect(self.rect(), 2, 2)

    def updateLinear(self):
        self.createPix()
        self.drawLinear()

    def updatePercentagePos(self,cur_x:int,cur_y:int):
        self.percentage_pos["x"] = round(cur_x / self.width(), 2)
        self.percentage_pos["y"] = round(cur_y / self.height(), 2)

    def sendEMitColor(self,x,y):
        self.updatePercentagePos(x, y)
        self.rgbaChange.emit(self.grab().toImage().pixelColor(x,y))

    def getColor(self) -> QColor:
        cursor = self.suppainter.virtualObj("cursor")
        return self.grab().toImage().pixelColor(*cursor.getPos())

    def heightTo2Discoloration(self,cursor,y):
        if y >= self.height() // 2:
            cursor.updateOpenAttr({"color": "white", "w": 2})
        else:
            cursor.updateOpenAttr({"color": "black", "w": 2})

    def mouseMoveEvent(self, e) -> None:
        if self.is_hide_hand is False:
            cursor = self.suppainter.virtualObj("cursor")
            x,y = e.x()-8,e.y()-8
            cursor.move(x,y)
            self.updatePercentagePos(x,y)
            self.heightTo2Discoloration(cursor,y)
            self.sendEMitColor(x,y)
            self.update()
        super().mouseMoveEvent(e)

    def mousePressEvent(self, e):
        if self.is_hide_hand is False:
            cursor = self.suppainter.virtualObj("cursor")
            data = [(2, 16),(3, 16),(4, 8),(5, 8)]
            for i,v in data:
                cursor.updateIndexToArgs(i, v)
            x, y = e.x()-8, e.y()-8
            cursor.move(x, y)
            self.heightTo2Discoloration(cursor, y)
            self.sendEMitColor(x,y)
            self.update()
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e:QResizeEvent):
        if self.is_hide_hand is False:
            cursor = self.suppainter.virtualObj("cursor")
            data = [(2, 10), (3, 10), (4, 5), (5, 5)]
            for i,v in data:
                cursor.updateIndexToArgs(i, v)
            self.update()
        super().mouseReleaseEvent(e)

    def paintEvent(self, e) -> None:
        self.suppainter.begin(self)
        self.suppainter.setRenderHints(qt.Antialiasing)
        rect = e.rect()
        self.suppainter.drawPixmap(rect, self.pix)
        self.suppainter.drawPixmap(rect, self.gray_pix)
        if self.is_hide_hand is False:
            self.suppainter.drawRoundedRect(20, 20, 10, 10, 5, 5, openAttr={"color": "black", "w": 2},
                                            virtualObjectName="cursor")
        self.suppainter.end()

    def resizeLinearPos(self,w,h):
        if self.suppainter.isVirtualObj("cursor"):
            cursor = self.suppainter.virtualObj("cursor")
            x = int(self.percentage_pos["x"] * w)
            y = int(self.percentage_pos["y"] * h)
            cursor.move(x,y)

    def resizeEvent(self, e) -> None:
        self.resizeLinearPos(e.size().width(),e.size().height())
        self.updateLinear()
        super().resizeEvent(e)

    def changeEvent(self, e:QWindowStateChangeEvent) -> None:
        if e.type() == e.WindowStateChange:
            if self.isMaximized() or self.isMinimized():
                self.resizeLinearPos(self.width(),self.height())
                self.updateLinear()
        super().changeEvent(e)


class PureColorWidget(QWidget):
    rgbaChange = Signal(QColor)
    hsvRgbaChange = Signal(QColor)

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(600,500)

        self.vlay = QVBoxLayout(self)

        self.colorbar = ColorBar()
        self.colorbar.setMinimumHeight(100)

        self.hsv = ColorHsv(self)
        self.hsv.setFixedHeight(50)
        self.hsv.rgbaChange.connect(self.colorbar.updateBgColor)

        self.vlay.addWidget(self.colorbar)
        self.vlay.addWidget(self.hsv)

        self.colorbar.rgbaChange.connect(self.rgbaChange.emit)
        self.hsv.rgbaChange.connect(self.hsvRgbaChange.emit)

    def setHideHand(self,b:bool):
        self.colorbar.setHideHand(b)

    def getColor(self)->QColor:
        return self.colorbar.getColor()

# ----------------------------以上是纯色图形的实现 --------------------------------


class GradientInfo:
    def __init__(self):
        self.info = {
            "pos": [],
            "pos_percentage":[],
            "gColor": {
                # handle_n is the specified naming rule
                "handle_1": {
                    "colorScope": 0,
                    "color": qt.red
                },
                "handle_2": {
                    "colorScope": 1,
                    "color": qt.blue
                }
            },
            # The linear gradient has only two handles
            "handle": [
                {
                    "vobj": "linear_1",
                    "handle": [],
                    "openAttr": {"c": "#000", "w": 2},
                    "brushAttr": {"c": qt.red},
                    "pos_percentage": {"x":0.0,"y":0.0}
                },
                {
                    "vobj": "linear_2",
                    "handle": [],
                    "openAttr": {"c": "#000", "w": 2},
                    "brushAttr": {"c": qt.blue},
                    "pos_percentage": {"x":0.0,"y":0.0}
                }
            ]
        }
        self.__max_hand_id = 2

    def getNewName(self)->str:
        self.addID()
        return "handle_{}".format(self.getID())

    def appendColor(self,colorScope,color):
        self.colors()[self.getNewName()]={
            "colorScope":colorScope,
            "color":color
        }

    def updateColorScope(self,hand_id,colorScope):
        self.colors()[hand_id]["colorScope"] = colorScope

    def updateColor(self,hand_id,color):
        self.colors()[hand_id]["color"] = color

    def delColor(self,hand_id):
        del self.colors()[hand_id]

    def addID(self):
        self.__max_hand_id+=1

    def getID(self) -> int:
        return self.__max_hand_id

    def colors(self) -> list:
        return self.info["gColor"]

    def pos(self) -> list:
        return self.info["pos"]

    def handles(self) -> list:
        return self.info["handle"]


class LinearInfo(GradientInfo):
    def __init__(self):
        super().__init__()
        self.info = {
            "pos": [0, 50, 330, 50],
            "pos_percentage": {"x":0.0,"y":0.0,"x1":0.0,"y1":0.0},
            "gColor": {
                # handle_n is the specified naming rule
                "handle_1": {
                    "colorScope": 0,
                    "color": qt.red
                },
                "handle_2": {
                    "colorScope": 1,
                    "color": qt.blue
                }
            },
            # The linear gradient has only two handles
            "handle": [
                {
                    "vobj": "linear_1",
                    "handle": [0, 50, 16, 16, 8, 40],
                    "openAttr": {"c": "#000", "w": 2},
                    "brushAttr": {"c": qt.red},
                    "pos_percentage":  {"x":0.0,"y":0.0}
                },
                {
                    "vobj": "linear_2",
                    "handle": [335, 50, 16, 16, 8, 40],
                    "openAttr": {"c": "#000", "w": 2},
                    "brushAttr": {"c": qt.blue},
                    "pos_percentage":  {"x":0.0,"y":0.0}
                }
            ]
        }


class RadialInfo(GradientInfo):
    def __init__(self):
        super().__init__()
        self.info = {
            "pos": [100, 100, 50, 80, 80],
            "pos_percentage": {"x":0.0,"y":0.0,"x1":0.0,"y1":0.0},
            "gColor": {
                # handle_n is the specified naming rule
                "handle_1": {
                    "colorScope": 0,
                    "color": qt.white
                },
                "handle_2": {
                    "colorScope": 1,
                    "color": qt.black
                }
            },
            "handle": [
                {
                    "vobj": "radial_1",
                    "handle": [100, 100, 16, 16, 8, 40],
                    "openAttr": {"c": "#000", "w": 2},
                    "brushAttr": {"c": qt.red},
                    "pos_percentage":  {"x":0.0,"y":0.0}
                },
                {
                    "vobj": "radial_2",
                    "handle": [80, 80, 14, 14, 7, 40],
                    "openAttr": {"c": "#000", "w": 2},
                    "brushAttr": {"c": qt.blue},
                    "pos_percentage":  {"x":0.0,"y":0.0}
                },
                {
                    "vobj": "radial_3",
                    "handle": [120, 80, 14, 14, 7, 40],
                    "openAttr": {"c": "#000", "w": 2},
                    "brushAttr": {"c": qt.gray},
                    "pos_percentage":  {"x":0.0,"y":0.0}
                }

            ]
        }


class ConicalInfo(GradientInfo):
    def __init__(self):
        super().__init__()
        self.info = {
            "pos": [108, 108, 90],
            "pos_percentage": {"x":0.0,"y":0.0},
            "gColor": {
                # handle_n is the specified naming rule
                "handle_1": {
                    "colorScope": 0,
                    "color": qt.red
                },
                "handle_2": {
                    "colorScope": 1,
                    "color": qt.blue
                }
            },
            "handle": [
                {
                    "vobj": "conical_1",
                    "handle": [100, 100, 16, 16, 8, 40],
                    "openAttr": {"c": "#000", "w": 2},
                    "brushAttr": {"c": qt.red},
                    "pos_percentage":  {"x":0.0,"y":0.0}
                },
                {
                    "vobj": "conical_2",
                    "handle": [80, 80, 14, 14, 7, 40],
                    "openAttr": {"c": "#000", "w": 2},
                    "brushAttr": {"c": qt.blue},
                    "pos_percentage":  {"x":0.0,"y":0.0}
                }

            ]
        }
# ----------------------------以上是绘制渐变图形所需的信息--------------------------------


class CodeQSS:
    def __init__(self, code_type="qss"):
        self.code_type = code_type
        self.grab_type = Handle_Linear
        self.grab_mode = "pad"

        self.structure = {
            "head": "qlineargradient({}",
            "pos": "",
            "color": ""
        }

    def setGrab(self, g_type):
        self.grab_type = g_type

    def setGrabMode(self, mode):
        self.grab_mode = mode

    def posArgc(self) -> str:
        return self.structure["pos"]

    def setColorCompound(self, g_color):
        temp = ""
        for hand_id, value in g_color.items():
            colorScope = value["colorScope"]
            color = QColor(value["color"])
            temp += "stop: {} rgba({},{},{},{}),".format(colorScope, *color.getRgb())
        self.structure["color"] = temp[:-1] + ");"

    def colorCompound(self) -> str:
        return self.structure["color"]

    def build(self) -> str:
        if self.grab_type != Handle_Conical:
            if self.grab_mode == QGradient.RepeatSpread:
                mode = "repeat"
            elif self.grab_mode == QGradient.ReflectSpread:
                mode = "reflect"
            else:
                mode = "pad"
            head = self.structure["head"].format("spread:" + mode + ",")
        else:
            head = self.structure["head"].format("")
        pos = self.posArgc() + ","
        return head + pos + self.colorCompound()


class CodeQSSLinear(CodeQSS):
    def __init__(self):
        super().__init__()

    def setPosArgc(self, x1, y1, x2, y2):
        t = "x1:{},y1:{},x2:{},y2:{}"
        self.structure["pos"] = t.format(x1, y1, x2, y2)


class CodeQSSRadial(CodeQSS):
    def __init__(self):
        super().__init__()
        self.structure["head"] = "qradialgradient({}"

    def setPosArgc(self, cx, cy, r, fx, fy):
        t = "cx:{},cy:{},radius:{},fx:{},fy:{}"
        self.structure["pos"] = t.format(cx, cy, r, fx, fy)


class CodeQSSConical(CodeQSS):
    def __init__(self):
        super().__init__()
        self.grab_type = Handle_Conical
        self.structure["head"] = "qconicalgradient({}"

    def setPosArgc(self, cx, cy, angle):
        t = "cx:{},cy:{},angle:{}"
        self.structure["pos"] = t.format(cx, cy, angle)
# ----------------------------以上是生成渐变QSS类--------------------------------


class GradientWidget(QWidget):
    qssed = Signal(str)

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.g_type = Handle_Linear
        self.ginfo = None # type:LinearInfo

        self.suppainter = SuperPainter()

        self.spread = QGradient.PadSpread

        self.is_hide_hand = False
        # 当前鼠标左键选择的句柄
        self.cur_click_hand = None  # type:VirtualObject

        self.code_obj = None

    def setCodeObj(self,code):
        self.code_obj = code

    def __updateAll(self):
        self.qssed.emit(self.qss())
        self.updateLayer()
        self.update()

    def updateColorScope(self,hand_id,colorScope):
        self.ginfo.updateColorScope(hand_id,colorScope)
        self.__updateAll()

    def updateColor(self,hand_id,color):
        self.ginfo.updateColor(hand_id,color)
        self.__updateAll()

    def appendColor(self,colorScope,color):
        self.ginfo.appendColor(colorScope,color)
        self.__updateAll()

    def delColor(self,hand_id):
        self.ginfo.delColor(hand_id)
        self.__updateAll()

    def setHideHand(self,b:bool):
        self.is_hide_hand = b

    def createPix(self):
        if not hasattr(self,"obj_pix"):
            self.obj_pix = QPixmap(self.size())
        else:
            self.obj_pix = self.obj_pix.scaled(self.size())
        self.obj_pix.fill(qt.transparent)

    def setSpread(self,spread):
        spread_dict = {
            "pad":QGradient.PadSpread,
            "repeat":QGradient.RepeatSpread,
            "reflect":QGradient.ReflectSpread
        }
        self.spread = spread_dict[spread]
        self.updateLayer()
        self.update()

    def drawLayer(self):
        type_dict = {
            Handle_Linear:QLinearGradient,
            Handle_Radial:QRadialGradient,
            Handle_Conical:QConicalGradient,
        }

        self.g_obj = type_dict[self.g_type](*self.ginfo.pos())
        if self.g_type in [Handle_Linear,Handle_Radial]:
            self.g_obj.setSpread(self.spread)

        for c in self.ginfo.colors().values():
            self.g_obj.setColorAt(c["colorScope"], c["color"])

        painter = QPainter(self.obj_pix)
        painter.setRenderHints(qt.Antialiasing)
        painter.setPen(qt.NoPen)
        painter.setBrush(self.g_obj)
        painter.drawRect(self.rect())

    def updateLayer(self):
        self.createPix()
        self.drawLayer()

    def updatePercentagePos(self,x,y):
        for hand in self.ginfo.handles():
            if hand["vobj"] == self.cur_click_hand.virName():
                hand["pos_percentage"]["x"] = round(x / self.width(), 2)
                hand["pos_percentage"]["y"] = round(y / self.height(), 2)

    def fckGradient(self,vname,e):
        # 具体的渐变实现交给子类
        pass

    def mouseMoveEvent(self, e) -> None:
        if self.cur_click_hand:
            r = self.cur_click_hand.getVirtualArgs()[4]  # radius
            x,y = e.x()-r,e.y()-r
            self.fckGradient(self.cur_click_hand.virName(),e)
            self.cur_click_hand.move(x,y)
            self.updatePercentagePos(x,y)
            self.updateLayer()
            self.update()
        self.qssed.emit(self.qss())
        super().mouseMoveEvent(e)

    def mousePressEvent(self, e) -> None:
        if e.buttons() == Qt.LeftButton and self.is_hide_hand is False:
            for hand in self.ginfo.handles():
                c_hand = self.suppainter.virtualObj(hand["vobj"])
                if c_hand.isClick(e):
                    c_hand.updateOpenAttr({"c": "#fff", "w": 3})
                    self.cur_click_hand = c_hand
                    r = c_hand.getVirtualArgs()[4]  # radius
                    x, y = e.x() - r, e.y() - r
                    self.updatePercentagePos(x,y)
                    break
            self.update()
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e) -> None:
        if self.is_hide_hand is False:
            for hand in self.ginfo.handles():
                c_hand = self.suppainter.virtualObj(hand["vobj"])
                if c_hand.isClick(e):
                    c_hand.updateOpenAttr({"c": "#000", "w": 2})
            self.cur_click_hand = None
            self.update()
        super().mouseReleaseEvent(e)

    def paintEvent(self, e) -> None:
        self.suppainter.begin(self)
        self.suppainter.setRenderHints(qt.Antialiasing)

        self.suppainter.drawPixmap(e.rect(),self.obj_pix)

        if self.is_hide_hand is False:
            for hand in self.ginfo.handles():
                rect = hand["handle"][:4]
                rs = hand["handle"][4],hand["handle"][4]
                self.suppainter.drawRoundedRect(*rect,*rs,openAttr=hand["openAttr"],
                                                brushAttr=hand["brushAttr"],
                                                virtualObjectName=hand["vobj"])

        self.suppainter.end()

    def fckResize(self,vname,pos,pos_dict):
        # 渐变随窗口的变化而变化,由子类实现
        pass

    def updateResize(self,w,h):
        for hand in self.ginfo.handles():
            vname = hand["vobj"]
            if self.suppainter.isVirtualObj(vname):
                c_hand = self.suppainter.virtualObj(vname)
                hand["handle"][0] = int(hand["pos_percentage"]["x"]*w)
                hand["handle"][1] = int(hand["pos_percentage"]["y"]*h)
                self.fckResize(vname,self.ginfo.pos(),self.ginfo.info["pos_percentage"])
                c_hand.move(*hand["handle"][:2])

    def resizeEvent(self, e) -> None:
        w, h = e.size().width(), e.size().height()
        self.updateResize(w,h)
        self.updateLayer()
        super().resizeEvent(e)

    def changeEvent(self, e:QWindowStateChangeEvent) -> None:
        if e.type() == e.WindowStateChange:
            if self.isMaximized() or self.isMinimized():
                self.updateResize(self.width(),self.height())
                self.updateLayer()
        super().changeEvent(e)

    def qss(self):
        self.code_obj.setGrab(self.g_type)
        self.code_obj.setGrabMode(self.spread)
        self.code_obj.setColorCompound(self.ginfo.colors())
        return self.code_obj.build()


class LinearWidget(GradientWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.g_type = Handle_Linear
        self.ginfo = LinearInfo()

        self.code = CodeQSSLinear()
        self.setCodeObj(self.code)

        self.updateLayer()

    def fckResize(self,vname,pos,pos_dict):
        if vname == "linear_1":
            pos[0] = int(pos_dict["x"]*self.width())
            pos[1] = int(pos_dict["y"]*self.height())
        elif vname == "linear_2":
            pos[2] = int(pos_dict["x1"]*self.width())
            pos[3] = int(pos_dict["y1"]*self.height())

    def fckGradient(self,vname,e):
        if vname == "linear_1":
            self.ginfo.pos()[0] = e.x()
            self.ginfo.pos()[1] = e.y()
            self.ginfo.info["pos_percentage"]["x"] = round(e.x()/self.width(),2)
            self.ginfo.info["pos_percentage"]["y"] = round(e.y()/self.height(),2)
        elif vname == "linear_2":
            self.ginfo.pos()[2] = e.x()
            self.ginfo.pos()[3] = e.y()
            self.ginfo.info["pos_percentage"]["x1"] = round(e.x() / self.width(), 2)
            self.ginfo.info["pos_percentage"]["y1"] = round(e.y() / self.height(), 2)

        x,y,x1,y1 = self.ginfo.pos()
        if x and y and x1 and y1:
            x,y = round(x/self.width(),3),round(y/self.height(),3)
            x1,y1 = round(x1/self.width(),3),round(y1/self.height(),3)
            self.code.setPosArgc(x,y,x1,y1)


class RadialWidget(GradientWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.g_type = Handle_Radial
        self.ginfo = RadialInfo()

        self.code = CodeQSSRadial()
        self.setCodeObj(self.code)

        self.updateLayer()

    def fckResize(self,vname,pos,pos_dict):
        if vname == "radial_1":
            pos[0] = int(pos_dict["x"] * self.width())
            pos[1] = int(pos_dict["y"] * self.height())
        elif vname == "radial_2":
            pos[3] = int(pos_dict["x1"] * self.width())
            pos[4] = int(pos_dict["y1"] * self.height())
        elif vname == "radial_3":
            pass

    def fckGradient(self,vname,e):
        if vname == "radial_1":
            self.ginfo.pos()[0] = e.x()
            self.ginfo.pos()[1] = e.y()
            self.ginfo.info["pos_percentage"]["x"] = round(e.x()/self.width(),2)
            self.ginfo.info["pos_percentage"]["y"] = round(e.y()/self.height(),2)
        elif vname == "radial_2":
            self.ginfo.pos()[3] = e.x()
            self.ginfo.pos()[4] = e.y()
            self.ginfo.info["pos_percentage"]["x1"] = round(e.x() / self.width(), 2)
            self.ginfo.info["pos_percentage"]["y1"] = round(e.y() / self.height(), 2)
        elif vname == "radial_3":
            x1,y1 = self.ginfo.pos()[:2]
            sqrt_n = int(math.sqrt(math.pow(e.x() - x1, 2) + math.pow(e.y() - y1, 2)))
            self.ginfo.pos()[2] = sqrt_n

        x, y, r, x1, y1 = self.ginfo.pos()
        if x and y and x1 and y1:
            r = round(r/ self.width(), 3)*2
            x,y = round(x/self.width(),3),round(y/self.height(),3)
            x1,y1 = round(x1/self.width(),3),round(y1/self.height(),3)
            self.code.setPosArgc(x,y,r,x1,y1)


class ConicalWidget(GradientWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.g_type = Handle_Conical
        self.ginfo = ConicalInfo()

        self.code = CodeQSSConical()
        self.setCodeObj(self.code)

        self.updateLayer()

    def fckResize(self,vname,pos,pos_dict):
        if vname == "conical_1":
            pos[0] = int(pos_dict["x"]*self.width())
            pos[1] = int(pos_dict["y"]*self.height())
        elif vname == "conical_2":
            pass

    def fckGradient(self,vname,e):
        if vname == "conical_1":
            self.ginfo.pos()[0] = e.x()
            self.ginfo.pos()[1] = e.y()
            self.ginfo.info["pos_percentage"]["x"] = round(e.x() / self.width(), 2)
            self.ginfo.info["pos_percentage"]["y"] = round(e.y() / self.height(), 2)
        elif vname == "conical_2":
            x,y = self.ginfo.pos()[:2]
            angle_arc = math.atan2(e.y() - y, e.x() - x)
            angle = angle_arc * 180 / math.pi
            self.ginfo.pos()[2] = -int(angle)

        x,y,a = self.ginfo.pos()
        if x and y:
            x,y = round(x/self.width(),3),round(y/self.height(),3)
            self.code.setPosArgc(x,y,a)

# ----------------------------以上是绘制渐变图形--------------------------------


HAND_ID_TYPE = str


class ColorOperation(QWidget):
    colorScoped = Signal(HAND_ID_TYPE,float)
    colored = Signal(HAND_ID_TYPE,QColor)
    newColored = Signal(float,QColor)
    delColored = Signal(HAND_ID_TYPE)

    def __init__(self,g_type="linear",*args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setFixedHeight(60)

        self.suppainter = SuperPainter()

        if g_type == Handle_Radial:
            handle1_color = qt.white
            handle2_color = qt.black
        else:
            handle1_color = qt.red
            handle2_color = qt.blue

        self.handles = [
            {
                "vobj": "handle_1",
                "handle": [5, 5, 20, 20, 10, 40],
                "openAttr": {"c": "#000", "w": 2},
                "brushAttr": {"c": handle1_color},
                "colorScope": 0,
                "color": handle1_color
            },
            {
                "vobj": "handle_2",
                "handle": [335, 5, 20, 20, 10, 40],
                "openAttr": {"c": "#000", "w": 2},
                "brushAttr": {"c": handle2_color},
                "colorScope": 1,
                "color": handle2_color
            }
        ]

        # The id only increases and does not decrease, maintaining uniqueness
        self.max_handle_id = 2

        self.g_type = g_type

        self.cursor_flag = None

        self._right_pressed_pos = QPoint(0,0)

        self.updatePix()

        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.menu_event)

    def menu_event(self):
        menu_ = QMenu(self)

        new_cursor = QAction("新建游标", self)
        del_cursor = QAction("删除游标", self)
        update_color = QAction("更新颜色", self)
        new_cursor.triggered.connect(self.createCursor_event)
        del_cursor.triggered.connect(self.delCursor_event)
        update_color.triggered.connect(self.updateCursorColor_event)
        menu_.addAction(new_cursor)
        menu_.addAction(del_cursor)
        menu_.addAction(update_color)

        menu_.popup(QCursor.pos())

    def getHandle(self, vname: str):
        for info in self.handles:
            if info["vobj"] == vname:
                return info
        return None

    def createCursor_event(self):
        if not self._right_pressed_pos:
            return

        x = self._right_pressed_pos.x()
        self.max_handle_id += 1

        structure = {
            "vobj": "handle_{}".format(self.max_handle_id),
            "handle": [x, 5, 20, 20, 10, 40],
            "openAttr": {"c": "#000", "w": 2},
        }

        color = QColorDialog.getColor()
        if color.isValid():
            v = round(1 / self.width() * x,2)
            structure["brushAttr"] = {"c": color}
            structure["colorScope"] = v
            structure["color"] = color
            self.handles.append(structure)
            self.newColored.emit(v,color)
            self.updatePix()
            self.update()
        self._right_pressed_pos = None

    def updateCursorColor_event(self):
        if not self._right_pressed_pos:
            return
        for vname in self.vObjs():
            cursor = self.suppainter.virtualObj(vname)
            if cursor.isClick(self._right_pressed_pos):
                color = QColorDialog.getColor()
                if color.isValid():
                    hand = self.getHandle(vname)
                    hand["brushAttr"]["c"] = color
                    hand["color"] = color
                    self.colored.emit(vname,color)
                    break
        self.updatePix()
        self.update()
        self._right_pressed_pos = None

    def delCursor_event(self):
        if not self._right_pressed_pos:
            return

        for vname in self.vObjs():
            cursor = self.suppainter.virtualObj(vname)
            if cursor.isClick(self._right_pressed_pos):
                self.handles.remove(self.getHandle(vname))
                self.delColored.emit(vname)
                break

        self.updatePix()
        self.update()
        self._right_pressed_pos = None

    def vObjs(self) -> list:
        return [vname["vobj"] for vname in self.handles]

    def updatePix(self):
        if not hasattr(self, "pix"):
            self.pix = QPixmap(self.size())
        else:
            self.pix = self.pix.scaled(self.size())
        self.pix.fill(qt.transparent)

        h2 = self.height() // 2
        linear = QLinearGradient(0, h2, self.width(), h2)

        for cursor in self.handles:
            linear.setColorAt(cursor["colorScope"], cursor["color"])

        painter = QPainter(self.pix)
        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform)
        painter.setPen(qt.NoPen)
        painter.setBrush(linear)
        painter.drawRoundedRect(self.rect(), 5, 5)

    def updateColorScope(self, cursor_name, v):
        for hand in self.handles:
            if hand["vobj"] == cursor_name:
                hand["colorScope"] = v
                break

    def mouseMoveEvent(self, e) -> None:
        if self.cursor_flag and e.buttons() == Qt.LeftButton:
            cursor = self.suppainter.virtualObj(self.cursor_flag)
            if e.x() + cursor.getWidth() // 2 >= cursor.getWidth() // 2 \
                    and e.x() <= self.width():
                cursor.updateIndexToArgs(0, e.x() - cursor.getWidth() // 2)
                v = round(1 / self.width() * e.x(),2)
                self.updateColorScope(self.cursor_flag, v)
                self.colorScoped.emit(self.cursor_flag, v)
                self.updatePix()
            self.update()
        super().mouseMoveEvent(e)

    def mousePressEvent(self, e) -> None:
        if e.buttons() == Qt.LeftButton:
            for vname in self.vObjs():
                cursor = self.suppainter.virtualObj(vname)
                if cursor.isClick(e):
                    color = QColor(cursor.getVirtualBrushAttr()["c"])
                    reverse_color = QColor(255 - color.red(), 255 - color.green(), 255 - color.blue())
                    cursor.updateOpenAttr({"c": reverse_color, "w": 3})
                    cursor.updateIndexToArgs(5, self.height()-10)
                    self.cursor_flag = vname
                    break
                else:
                    self.cursor_flag = ""
            self.update()
        elif e.buttons() == Qt.RightButton:
            self._right_pressed_pos = QPoint(e.pos().x(), e.pos().y())
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e) -> None:
        self.cursor_flag = ""
        for vname in self.vObjs():
            cursor = self.suppainter.virtualObj(vname)
            cursor.updateOpenAttr({"c": "#000", "w": 2})
            cursor.updateIndexToArgs(5, 40)
        self.update()
        super().mouseReleaseEvent(e)

    def paintEvent(self, e) -> None:
        self.suppainter.begin(self)
        self.suppainter.setRenderHints(qt.Antialiasing)

        self.suppainter.drawPixmap(e.rect(),self.pix)

        for hand in self.handles:
            rect = hand["handle"][:4]
            rs = hand["handle"][4]
            line_h =hand["handle"][5]
            self.suppainter.drawCursor(*rect,rs,line_h,
                                        openAttr=hand["openAttr"],
                                        brushAttr=hand["brushAttr"],
                                        virtualObjectName=hand["vobj"]
                                       )

        self.suppainter.end()

    def resizeEvent(self, e: QResizeEvent) -> None:
        width = e.size().width()
        for hand in self.handles:
            if self.suppainter.isVirtualObj(hand["vobj"]):
                cursor = self.suppainter.virtualObj(hand["vobj"])
                new_pos = int(width * hand["colorScope"])
                if new_pos >= width:
                    new_pos -= 20
                cursor.updateIndexToArgs(0, new_pos)
        self.updatePix()
        super().resizeEvent(e)

# ----------------------------渐变图形 通用的操作台--------------------------------


class CombinationFigure(QWidget):
    qssed = Signal(str)

    def __init__(self,g_type,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(600,500)
        self.vlay = QVBoxLayout(self)

        if g_type == Handle_Linear:
            self.widget = LinearWidget()
        elif g_type == Handle_Radial:
            self.widget = RadialWidget()
        elif g_type == Handle_Conical:
            self.widget = ConicalWidget()
        else:
            raise Exception("no!")

        self.cop = ColorOperation()

        self.vlay.addWidget(self.widget)
        self.vlay.addWidget(self.cop)

        self.widget.qssed.connect(self.qssed.emit)

        self.cop.colorScoped.connect(self.widget.updateColorScope)
        self.cop.colored.connect(self.widget.updateColor)
        self.cop.newColored.connect(self.widget.appendColor)
        self.cop.delColored.connect(self.widget.delColor)

    def setHideHand(self,b:bool):
        self.widget.setHideHand(b)

    def setSpread(self,spread):
        self.widget.setSpread(spread)

    def getQSS(self) -> str:
        return self.widget.qss()


class Linear(CombinationFigure):
    def __init__(self,*args,**kwargs):
        super().__init__(Handle_Linear,*args,**kwargs)


class Radial(CombinationFigure):
    def __init__(self,*args,**kwargs):
        super().__init__(Handle_Radial,*args,**kwargs)


class Conical(CombinationFigure):
    def __init__(self,*args,**kwargs):
        super().__init__(Handle_Conical,*args,**kwargs)



# -*- coding:utf-8 -*-
# @time:2023/3/249:19
# @author:LX
# @file:1.py
# @software:PyCharm
import sys

from PyQt5.QtCore import QSize, QPoint,QRect
from PyQt5.QtGui import QColor
from PyQt5.QtWidgets import QWidget, QApplication, QPushButton
from PyQtGuiLib.animation import Animation


class Test(QWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.resize(800, 600)

        self.btn = QPushButton("按钮", self)
        self.btn.move(50, 50)
        self.btn.resize(100, 60)
        self.btn.setStyleSheet('''
                QPushButton{
                    background-color:rgb(50,100,200);
                    color:rgb(255,0,0);
                }
                ''')

        self.start_btn = QPushButton("开始动画", self)
        self.start_btn.setGeometry(500, 50, 100, 60)
        self.start_btn_2 = QPushButton("修改动画", self)
        self.start_btn_2.setGeometry(500, 120, 100, 60)

        # 实例化动画类
        self.ani = Animation()
        # 设置动画时长
        self.ani.setDuration(2000)  # 2秒

        # 再添加一个移动的动画
        s = [(253, 298), (255, 298), (259, 300), (263, 303), (268, 309),
             (274, 317), (281, 326), (286, 336), (292, 346), (298, 352), (299, 354), (303, 359),
             (307, 365), (309, 368), (310, 369), (311, 371), (314, 371), (317, 371), (324, 371), (332, 369),
             (343, 366), (355, 360), (372, 352), (392, 344), (415, 335), (435, 326), (454, 319), (475, 310),
             (490, 303), (505, 295), (515,288), (525, 283), None]

        self.v = self.ani.createAniNumbers(253,298)

        # self.ani.addValuesAni({
        #     "propertyName":"value",
        #     "sv":self.v,
        #     "ev":
        # })


        # self.ani.start()





if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = Test()
    win.show()
    sys.exit(app.exec())



# -*- coding:utf-8 -*-
# @time:2023/4/717:36
# @author:LX
# @file:eg4.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QColor,
    QPushButton,
    QPoint
)
from PyQtGuiLib.styles import SuperPainter


class Test(QWidget):
    def __init__(self):
        super().__init__()
        self.resize(800,600)
        '''
            SuperPainter第四个案例,
            
            在上一个案例,了解了什么是 虚拟对象,这个案例
            将教你如何使用虚拟对象
        '''
        self.painter = SuperPainter()  # 创建一个画师对象,注意这个对象不要创建在paintEvent里面

        # 这个创建一个按钮,等下点击这个按钮来修改图形
        self.btn = QPushButton("修改图形",self)
        self.btn.move(200,50)
        self.btn.clicked.connect(self.updateDraw)

        self.setMouseTracking(True)
        self.cupos = QPoint(-1,-1)

    def mousePressEvent(self, e):
        myrect = self.painter.virtualObj("myrect")
        myrect2 = self.painter.virtualObj("myrect2")
        print("是否点击在myrect图形上",myrect.isClick(self.cupos))
        print("是否点击在myrect2图形上",myrect2.isClick(self.cupos))
        super().mousePressEvent(e)

    def mouseMoveEvent(self, e):
        self.cupos = e.pos()
        myrect2 = self.painter.virtualObj("myrect2")
        if myrect2.isClick(self.cupos):
            myrect2.updateOpenAttr(openAttr={"color": "red", "width": 3})
        else:
            myrect2.updateOpenAttr(openAttr={"color": "green", "width": 3})
        self.update()
        super().mouseMoveEvent(e)

    # 修改图形事件
    def updateDraw(self):
        '''
            所有与虚拟对象有关的方法都在 self.painter.virtualObj() 这个下面

            几乎所有的虚拟对象方法的第一个参数都是 "虚拟对象名称",就像self一样

            这里用的两个方法:
                getVirtualArgs() 获取当前图形的位置大小等信息
                updateArgs() 修改当前图形的位置大小等信息
                updateOpenAttr() 修改当前图形的属性
        '''
        myrect=self.painter.virtualObj("myrect")
        myrect.updateArgs(20,20,100,100)
        myrect.updateOpenAttr(openAttr={"color":"red","width":3})
        myrect.updateBrushAttr(brushAttr={"color":QColor("green")})


        myrect2=self.painter.virtualObj("myrect2")
        myrect2.scale(1.2)
        # myrect2.setHide(True) # True隐藏图形
        # myrect2.move(300,300)
        # myrect2.updateOpenAttr(openAttr={"color":"red","width":3})
        # myrect2.updateBrushAttr(brushAttr={"color":QColor("green")})

        self.update() # 这里必须调用一下,来刷新图形

    def paintEvent(self, QPaintEvent):
        self.painter.begin(self)

        # 创建一个 图形虚拟对象
        self.painter.drawRect(20, 20, 50, 50,openAttr={"color":"green"}, virtualObjectName="myrect")
        self.painter.drawRect(200, 200, 50, 50,openAttr={"color":"green"}, virtualObjectName="myrect2")
        self.painter.drawRoundedRect(10,300,150,80,openAttr={"c":"#00557f"})
        self.painter.end()


if __name__ == '__main__':
    app = QApplication(sys.argv)

    win = Test()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



# -*- coding:utf-8 -*-
# @time:2023/3/1715:33
# @author:LX
# @file:animationDrawType.py
# @software:PyCharm

'''
    这个类为绘图动画的封装类型
'''
import typing
from PyQtGuiLib.header import (
    QColor,
    QRect,
    QPoint
)

Any = typing.Any
Color = typing.TypeVar("Color",int,QColor)
Rect = typing.TypeVar("Rect",int,QRect)
Point = typing.TypeVar("Point",int,QPoint)

# 值类型
class AniNumber:
    def __init__(self,n:Any):
        self.__n = n

    def setNumber(self,n:Any):
        self.__n = n

    def value(self)->Any:
        return self.__n

    # 这里表示任意类型
    def type(self) -> Any:
        return type(self.value())

# 多值类型
class AniNumbers:
    def __init__(self,*args):
        self.__an = []
        if args:
            self.setNumbers(*args)

    def setNumbers(self,*args):
        for an in args:
            self.__an.append(AniNumber(an))

    def type(self) -> list:
        return list

    def numberObjs(self) -> list:
        return [n for n in self.__an]

    def values(self) -> list:
        return [n.value() for n in self.__an]


# 颜色类型
class AniColor(AniNumbers):
    def __init__(self,r:Color,g:int,b:int,a:int=255):
        if isinstance(r,QColor):
            value = QColor(*r.getRgb())
        elif 0 <= r and r <= 255 and\
             0 <= g and g <= 255 and\
             0 <= b and b <= 255 and\
             0 <= a and a <= 255:
                value = r,g,b,a
        else:
            raise Exception("The value ranges from 0 to 255!")
        super().__init__(*value)

    def type(self) -> QColor:
        return QColor

    def values(self) -> QColor:
        return QColor(*super().values())


# 矩形类型
class AniRect(AniNumbers):
    def __init__(self,x:Rect,y:int,w:int,h:int):
        if isinstance(x,QRect):
            value = x.getRect()
        else:
            value = x,y,w,h
        super().__init__(*value)

    def type(self) -> QRect:
        return QRect

    def values(self) -> QRect:
        return QRect(*super().values())

# 点类型
class AniPoint(AniNumbers):
    def __init__(self, x:Point, y: int):
        if isinstance(x, QPoint):
            value = x.x(),x.y()
        else:
            value = x, y
        super().__init__(*value)

    def type(self) -> QPoint:
        return QPoint

    def values(self) -> QPoint:
        return QPoint(*super().values())


# 阴影类型(测试中)
class AniShadow(AniNumbers):
    def __init__(self,offx:int,offy:int,r:int,color:QColor):
        super().__init__(offx,offy,r,color)

    def type(self) -> str:
        return "shadow"

    def values(self) -> list:
        return [*super().values()]


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,sys,
    Signal,
    Widget,
    QPainter,
    QColor,
    QBrush,
    QPen,
    qt,
    QRect,
    QSize,
    pyqtProperty,
    QMouseEvent
)


class Slider(Widget):
    valueChanged = Signal(int)

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(600,80)

        # 边距
        self.margin = 9

        # 背景高度
        self.__bgH = 8

        # 图标位置/大小,百分比位置
        self.icon_x = self.leftMargin()
        self.icon_size = QSize(15,15)
        self.icon_pre_pos = 0

        # 流动层
        self.flow_w = 0

        # 缓冲层,百分比位置
        self.buff_w = 0
        self.buff_pre_pos = 0

        # 最大值
        self.max_value = 100

        # -----
        self.__bgColor = QColor(200,200,200)
        self.__buffColor = QColor(100,100,100)
        self.__flowColor = QColor(200,100,200)
        self.__iconColor = QColor(230, 40, 50)
        # -----

        # 返回值保存
        self.re_flow_v = 0
        self.re_buff_v = 0

        # 图标是否具有 hover 效果
        self.__isHoverStart = False
        self.__isHover = False

    # ----------------------------
    def __set_bgColor(self,color:QColor):
        self.__bgColor = color

    def get_bgColor(self)->QColor:
        return self.__bgColor

    def __set_buffColor(self, color: QColor):
        self.__buffColor = color

    def get_buffColor(self) -> QColor:
        return self.__buffColor

    def __set_flowColor(self, color: QColor):
        self.__flowColor = color

    def get_flowColor(self) -> QColor:
        return self.__flowColor

    def __set_iconColor(self, color: QColor):
        self.__iconColor = color

    def get_iconColor(self) -> QColor:
        return self.__iconColor

    def __set_iconSize(self,size:int):
        self.icon_size = QSize(size,size)
    
    def __set_bgHeight(self,h:int):
        self.__bgH = h

    # ----------------------------

    def setHoverIcon(self,b:bool):
        self.__isHoverStart = b

    def getMaxValue(self) -> int:
        return self.max_value

    def setMaxValue(self,v):
        self.max_value = v

    # 设置当前值
    def setValue(self,v:int):
        if not v:
            self.buff_w = 0
            return

        x = self.valueToPos(v)
        self.icon_pre_pos = (x - self.leftMargin()) / (self.width() - self.rightMargin())
        self.flow_w = self.valueToPos(v)+self.icon_size.width()//2

    def getValue(self) -> int:
        if not self.flow_w:
            return 0

        return int(self.posToValve(self.flow_w-self.icon_size.width()//2))

    # 设置缓冲值
    def setBuffValue(self,v:int):
        if not v:
            self.buff_w = 0
            return

        x = self.valueToPos(v)
        self.buff_pre_pos = (x - self.leftMargin()) / (self.width() - self.rightMargin())
        self.buff_w = self.valueToPos(v)

    def getBuffValue(self)->int:
        if not self.flow_w:
            return 0

        return int(self.posToValve(self.buff_w))

    def leftMargin(self) -> int:
        return self.margin

    def rightMargin(self) -> int:
        return self.leftMargin() << 1

    def centerH(self) -> int:
        return self.height()//2 - self.__bgH//2

    # 坐标值 转 百分比值
    def posToValve(self,x:int) -> float:
        '''
            总宽度: self.width()  当前坐标值:x
            最大值 getMaxValue()  ?
        '''
        w = self.width()-self.rightMargin()
        return x/w*self.getMaxValue()

    # 百分比 转 坐标值
    def valueToPos(self,v:int) -> float:
        '''
            总宽度: self.width()  ?
            最大值 getMaxValue()  当前百分比值 v
        '''
        w = self.width()-self.rightMargin()
        return v/self.getMaxValue()*w

    def __mouse(self,event:QMouseEvent):
        x = event.x()
        w = self.width() - self.leftMargin()
        if x >= self.leftMargin() and x <= w:
            # 记录图标的百分比值
            self.icon_pre_pos = (x-self.leftMargin())/(self.width()-self.rightMargin())
            # print("输出值:",self.posToValve(x - self.leftMargin()))
            self.valueChanged.emit(int(self.posToValve(x - self.leftMargin())))
            # 更新流动层
            self.flow_w = x + self.icon_size.width()//2
            # 图标位置
            self.icon_x = x
            self.update()

    def leaveEvent(self, e) -> None:
        if self.__isHoverStart:
            self.__isHover = False
            self.update()

    def enterEvent(self, e) -> None:
        if self.__isHoverStart:
            self.__isHover = True
            self.update()

    def mousePressEvent(self, event:QMouseEvent) -> None:
        self.__mouse(event)

    def mouseMoveEvent(self, event:QMouseEvent) -> None:
        self.__mouse(event)

    def __drawRoundedRect(self,painter:QPainter,brush_color:QColor,w:int):
        bru = QBrush(brush_color)
        painter.setBrush(bru)
        if w - self.rightMargin() < 0:
            rect = QRect(self.leftMargin(), self.centerH(), w, self.__bgH)
        else:
            rect = QRect(self.leftMargin(), self.centerH(), w - self.rightMargin(), self.__bgH)

        painter.drawRoundedRect(rect,self.get_radius(), self.get_radius())

    # 绘制图标
    def drawIcon(self,painter:QPainter):
        if self.__isHoverStart is False or self.__isHover:
            bru = QBrush(self.get_iconColor())
            painter.setBrush(bru)

            icon_w = self.icon_size.width()
            icon_h = self.icon_size.height()
            y = self.height()//2-icon_w//2
            rect = QRect(self.icon_x-icon_w//2,
                         y,
                         icon_w,
                         icon_h)
            painter.drawRoundedRect(rect,icon_w//2,icon_h//2)

    # 流动层
    def drawFlowColor(self,painter:QPainter):
        self.__drawRoundedRect(painter,self.get_flowColor(),
                               int(self.flow_w))

    # 缓冲层
    def drawBuffBgColor(self,painter:QPainter):
        self.__drawRoundedRect(painter,self.get_buffColor(),
                               int(self.buff_w))

    # 底色
    def drawBackgroundColor(self,painter:QPainter):
        self.__drawRoundedRect(painter,self.get_bgColor(),
                               self.width())

    def paintEvent(self, event) -> None:
        painter = QPainter(self)
        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform)

        painter.setPen(qt.NoPen)

        self.__update()
        # ------------------------------
        self.drawBackgroundColor(painter)
        self.drawBuffBgColor(painter)
        self.drawFlowColor(painter)
        self.drawIcon(painter)

        painter.end()

    # 更新
    def __update(self):
        # 更新缓冲层
        self.buff_w = int(self.buff_pre_pos*(self.width()-self.rightMargin()))+self.leftMargin()
        # 更新图标位置
        self.icon_x = int(self.icon_pre_pos*(self.width()-self.rightMargin()))+self.leftMargin()
        # 更新流动层
        if self.flow_w:
            self.flow_w = self.icon_x + self.icon_size.width()//2

    # 自定义属性
    bgColor = pyqtProperty(QColor,fset=__set_bgColor,fget=get_bgColor)
    buffColor = pyqtProperty(QColor,fset=__set_buffColor,fget=get_buffColor)
    flowColor = pyqtProperty(QColor,fset=__set_flowColor,fget=get_flowColor)
    iconColor = pyqtProperty(QColor,fset=__set_iconColor,fget=get_iconColor)
    iconSize = pyqtProperty(int,fset=__set_iconSize,fget=None)
    bgHeight = pyqtProperty(int,fset=__set_bgHeight,fget=None)

# if __name__ == '__main__':
#     app = QApplication(sys.argv)
#
#     win = Slider()
#     win.show()
#     win.setMaxValue(200)
#     win.setValue(100)
#     print(win.getValue())
#     # win.setBuffValue(46)
#     # print(win.getBuffValue())
#     win.setStyleSheet('''
# qproperty-radius:4;
# qproperty-flowColor:rgb(100,50,50);
# qproperty-iconSize:15;
#     ''')
# #     win.setStyleSheet('''
# # qproperty-radius:4;
# # qproperty-backgroundColor:rgb(200, 200, 200);
# # qproperty-sliderH:8;
# #     ''')
#
#     if PYQT_VERSIONS in ["PyQt6","PySide6"]:
#         sys.exit(app.exec())
#     else:
#         sys.exit(app.exec_())


import platform
import math

from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    DesktopWidget,
    QApplication,
    QPoint,
    QSize,
    QFontMetricsF,
    QFont,
    QWidget,
    CustomStyle
)

'''
QListWidget 在Pyqt5页面刷新的方式 update(),其他版本可以用repaint()来达到一样的效果
QListWidget 的update(item)需要传递一个参数
'''

is_win_sys = True if platform.system() in ["win32","Windows"] else False

is_mac_sys = True if platform.system() == "Darwin" else False


'''
    These several screen-related methods can only be called in the window,
    otherwise an error will be reported
'''


IS_Qt5 = PYQT_VERSIONS == "PyQt5"
IS_QtIt = PYQT_VERSIONS in ["PyQt6","PySide2","PySide6"]


def desktopCount() -> int:
    return len(DesktopWidget.screens())


# Gets the size of a single desktop
def desktopSize() -> QSize:
    return DesktopWidget.primaryScreen().size()


# Total size of all screens
def desktopAllSize() ->QSize:
    size = desktopSize()
    return QSize(size.width()*desktopCount(),size.height())


# The desktop is in the middle
def desktopCenter(parent) -> QPoint:
    center = DesktopWidget.primaryScreen().availableGeometry().center()
    return QPoint(center.x()-parent.width()//2,center.y()-parent.height()//2)


# 获取文字大小
def textSize(font:QFont,text:str) -> QSize:
    fs = QFontMetricsF(font)
    if IS_Qt5:
        return QSize(int(fs.width(text)),int(fs.height()))
    elif IS_QtIt:
        return QSize(int(fs.horizontalAdvance(text)+1), int(fs.height()+1)) # +1 是为了补偿丢失的像素
    else:
        return QSize(0,0)


# RGB 转 HSV
def rgbTohsv(r, g, b):
    r, g, b = r/255.0, g/255.0, b/255.0
    mx,mn = max(r, g, b),min(r, g, b)
    df = mx-mn
    h = 0
    if mx == mn:
        h = 0
    elif mx == r:h = (60 * ((g-b)/df) + 360) % 360
    elif mx == g:h = (60 * ((b-r)/df) + 120) % 360
    elif mx == b:h = (60 * ((r-g)/df) + 240) % 360

    s = 0 if mx ==0 else df/mx

    v = mx
    return h, s, v

# 加载UI文件
def loadUic(ui_path:str) -> QWidget:
    if PYQT_VERSIONS in ["PyQt5","PyQt6"]:
        from PyQtGuiLib.header import uic
        return uic.loadUi(ui_path)
    elif PYQT_VERSIONS in ["PySide2","PySide6"]:
        from PyQtGuiLib.header import QFile,QUiLoader
        uif = QFile(ui_path)
        loader = QUiLoader()
        ui = loader.load(uif)
        uif.close()
        return ui



# HSV 转 RGB
def hsvTorgb(h, s, v):
    h,s,v = float(h),float(s),float(v)

    h_60 = h / 60.0
    h_60f = math.floor(h_60)
    hi = int(h_60f) % 6
    f = h_60 - h_60f

    p,q,t = v * (1 - s),v * (1 - f * s),v * (1 - (1 - f) * s)
    r, g, b = 0, 0, 0

    if hi == 0: r, g, b = v, t, p
    elif hi == 1: r, g, b = q, v, p
    elif hi == 2: r, g, b = p, v, t
    elif hi == 3: r, g, b = p, q, v
    elif hi == 4: r, g, b = t, p, v
    elif hi == 5: r, g, b = v, p, q
    r, g, b = int(r * 255), int(g * 255), int(b * 255)
    return r, g, b


class Widget(QWidget,CustomStyle):
    def __init__(self,*args,**kwargs):
        if PYQT_VERSIONS in ["PySide6", "PySide2"]:
            QWidget.__init__(self, *args, **kwargs)
            CustomStyle.__init__(self, *args, **kwargs)
        else:
            super().__init__(*args, **kwargs)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = Widget()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2023/2/818:14
# @author:LX
# @file:test_QssStyleAnalysis.py
# @software:PyCharm


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    qt,
    QPushButton,
    QLabel,
    Qt,
    QMainWindow,
    QPainterPath
)

from PyQtGuiLib.styles import QssStyleAnalysis


class Test(QMainWindow):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(600,600)

        self.l = QLabel("测试标签",self)
        self.btn = QPushButton("测试按钮",self)

        # QSS 解析器
        # self.qss = QssStyleAnalysis(self)
        # self.qss.setQSS('''
        # QPushButton{
        # color: rgb(0, 255, 127);
        # background-color:rgb(0, 170, 0);
        # }
        # QPushButton:hover{
        # }
        # ''')
        # self.qss.appendQSSDict({
        #     "QLabel":{
        #         "border":"2px solid yellow",
        #         "background-color":"rgb(150,20,200)",
        #     },
        #     "QLabel:hover": {
        #         "border": "2px solid blue",
        #         "color": "red",
        #     }
        # })
        # self.qss.selector("QPushButton").updateAttr("background-color", "rgb(255,0,0)")
        # # self.qss.selectorPoint("QPushButton.color")

        self.l.move(30, 30)
        self.btn.resize(150,80)
        self.btn.move(80,80)

        # 创建一个针对整个窗口的 QSS 解析器
        self.qss = QssStyleAnalysis(self)
        # 对窗口上所有按钮,标签设置样式
        self.qss.setQSS('''
                QPushButton{
                color: rgb(0, 255, 127);
                background-color:rgb(0, 170, 0);
                }
                QLabel{
                color: rgb(42, 55, 127);
                background-color:rgb(255, 170, 0);
                }
                ''')
        self.qss.selector("QPushButton").updateAttr("background-color", "red")  #
        print(self.styleSheet())


if __name__ == '__main__':
    app = QApplication(sys.argv)

    win = Test()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2023/4/717:30
# @author:LX
# @file:eg3.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QColor
)
from PyQtGuiLib.styles import SuperPainter



class Test(QWidget):
    def __init__(self):
        super().__init__()
        '''
            SuperPainter第三个案例,
            这里 将一个全新的概念 虚拟对象,
            
            那为什么需要 虚拟对象 呢?
            原本通过QPainter绘制的图形,当你想要动态修改的时候会发现很困难,无从下手,
            那如果图形是一个对象呢,对象是具有属性的,当我想要修改图形时,只需要修改对应图形的属性就好.
            这就是 虚拟对象 存在的意义
        '''
        self.painter = SuperPainter() # 创建一个画师对象,注意这个对象不要创建在paintEvent里面

    def paintEvent(self, QPaintEvent):
        self.painter.begin(self)

        '''
            将一个图形变成虚拟对象的方法很简单,需要用到一个关键字参数
            virtualObjectName,给这个参数给一个虚拟对象名称就完成了,
            同时这句绘图代码,也可以称为 图形虚拟对象的初始化
        '''
        self.painter.drawRect(20,20,50,50,virtualObjectName="myrect")

        self.painter.end()

if __name__ == '__main__':
    app = QApplication(sys.argv)

    win = Test()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



# -*- coding:utf-8 -*-
# @time:2023/3/3117:02
# @author:LX
# @file:logSizeTh.py
# @software:PyCharm

'''
    读取日志大小线程
'''

import os
from PyQtGuiLib.header import (
    QThread,
    Signal
)


# 计算日志大小线程
class LogSizeTh(QThread):
    Sizeed = Signal(int)

    def __init__(self, path:str):
        super().__init__()
        self.__path = path

    def setPath(self,path):
        self.__path = path

    def run(self):
        st = os.stat(self.__path)
        self.Sizeed.emit(st.st_size)



import re
import typing
from PyQtGuiLib.header import (
    QPropertyAnimation,
    QObject,
    QColor,
    Signal,
    QGraphicsDropShadowEffect
)
from PyQtGuiLib.styles import QssStyleAnalysis
from abandonCase.animation.animationDrawType import AniNumber
'''
    QPropertyAnimation: you're trying to animate a non-existing property value of your QObject
    该警告出现mac下,部分win可能也会出现,可以忽略该警告
'''


# 公用动画基类
class PropertyAnimation(QPropertyAnimation):
    drawfinished = Signal()

    Parallel = 1
    Sequential = 2
    # 动画对象模式
    Control = "control"  # 动画作用在普通控件上面
    Draw = "draw"  # 动画作用在绘制的图形上面

    def __init__(self,parent:QObject,ani_data:dict,ani_obj_mode="control"):
        self.__parent = parent
        self.attr = ani_data["propertyName"].decode()
        super().__init__(parent)

        # 动画对象模式
        self.__ani_obj_mode = ani_obj_mode

        self.__all_ani_data = ani_data

        self.aniInit()

        # 保存绘图动画的开始对象
        self.__drawStartObj = self.sv

    def allAniDatas(self)->dict:
        return self.__all_ani_data

    def aniInit(self):
        self.targetObj = self.allAniDatas().get("targetObj", None)
        self.propertyName = self.allAniDatas().get("propertyName", None)
        self.sv = self.allAniDatas().get("sv", None)
        self.atv = self.allAniDatas().get("atv", None)
        self.ev = self.allAniDatas().get("ev", None)
        self.call = self.allAniDatas().get("call", None)
        self.call_argc = self.allAniDatas().get("argc", None)
        self.selector = self.allAniDatas().get("selector", None)
        self.qssSuffix = self.allAniDatas().get("qss-suffix", "px")

        special = self.allAniDatas().get("special", self.special())
        duration = self.allAniDatas().get("duration", self.duration())
        loopCount = self.allAniDatas().get("loop", self.loopCount())

        self.setTargetObject(self.targetObj)
        self.setDuration(duration)
        self.setSpecial(special)
        self.setLoopCount(loopCount)

    def drawSv(self):
        return self.__drawStartObj

    def aniObjMode(self):
        return self.__ani_obj_mode

    # 设置动效
    def setSpecial(self,special):
        self.setEasingCurve(special)

    def special(self):
        return self.easingCurve()

    # 判断sv是不是this
    def isThis(self) -> bool:
        if isinstance(self.sv,str) and self.sv.lower() == "this":
            return True
        return False

    # 判断是否开启特殊动画
    def isEffect(self) -> bool:
        return True if self.allAniDatas().get("isEffect",None) else False

    # 返回备注信息
    def commentInfo(self) -> str:
        return self.allAniDatas().get("comment","")

    # 刷新绘图动画
    def updateDraw(self):
        self.__parent.update()

    # 更新动画信息
    def updateAni(self,new_datas:dict):
        self.allAniDatas().update(new_datas)
        self.aniInit()
        self.createAni()

    # 子类重写(默认模式)
    def createAni(self):
        self.setStartValue(self.sv)
        if self.atv:
            one_e = self.atv[0]
            if isinstance(one_e, tuple) or isinstance(one_e, list):
                for step, value in self.atv:
                    self.setKeyValueAt(step, value)
            else:
                mean_time = 1 / len(self.atv)  # 平均时间
                step = 0.0
                for value in self.atv:
                    step += mean_time
                    self.setKeyValueAt(step, value)
        self.setEndValue(self.ev)

        self.aniCall()

    # 执行动画的回调函数
    def aniCall(self):
        if self.call:
            if self.call_argc:
                self.finished.connect(lambda: self.call(self.targetObj, *self.call_argc))
            else:
                self.finished.connect(lambda: self.call(self.targetObj))


# 与QSS属性动画相关的基类
class QSSPropertyAnimation(PropertyAnimation):
    def __init__(self,parent:QObject,ani_data:dict,ani_obj_mode="control"):
        super().__init__(parent,ani_data,ani_obj_mode)

        if self.selector is None:
            raise Exception("backgroundColor The selector must be provided!")

        self.setPropertyName(self.propertyName)

        self.qss = QssStyleAnalysis(self.targetObj)
        self.qss.setQSS(self.targetObj.styleSheet())
        self.createAni()

    def setStartValue(self, value) -> None:
        '''
            当这个值是 this 的时候,指向自己已经编写的属性值,
            如果没有这个属性值,将报错,
            如果这个属性中包含 color 则判定为颜色属性,否则判断为数值形
            如果这个属性值中包含px,pt 这里的,直接返回数值形
        :param value:
        :return:
        '''
        if self.isThis():
            attr = self.attr.lower()
            if self.qss.selector(self.selector).isAttr(attr):
                attrValue = self.qss.selector(self.selector).attr(self.attr) # Gets the value of the property
                if "color" in attr:
                    color = attrValue
                    if "rgb" in color.lower() or "rgba" in color.lower():
                        color_list=[int(i) for i in re.findall(r"[0-9]{1,3}",color)]
                        value = QColor(*color_list)
                    else:
                        value = QColor(color)
                elif "px" in attrValue or "pt" in attrValue:
                    value = int(re.findall(r"[0-9]+",attrValue)[0])
            else:
                raise Exception("The sv attribute is this, but QSS does not have this attribute!,{}".format(self.attr))

        super().setStartValue(value)

    def updateState(self, newState, oldState) -> None:
        super().updateState(newState,oldState)

    def interpolated(self, from_, to, progress: float):
        return super().interpolated(from_,to,progress)

    def __toValue(self,value):
        if isinstance(value, int):
            value = "{}{}".format(value,self.qssSuffix)
        elif isinstance(value, QColor):
            value = "rgba({},{},{},{})".format(*value.getRgb())

        else:
            raise Exception("This type is not supported yet.{}".format(value))
        return value

    def updateCurrentValue(self, value) -> None:
        self.qss.appendQSS(self.targetObj.styleSheet())
        self.qss.selector(self.selector).updateAttr(self.propertyName.decode(), self.__toValue(value))

# --------------


# 通用普通控件动画
class AnimationControl(PropertyAnimation):
    def __init__(self,parent:QObject,ani_data:dict,ani_obj_mode="control"):
        super().__init__(parent,ani_data,ani_obj_mode)

        self.setPropertyName(self.propertyName)
        self.createAni()

    def setStartValue(self, value) -> None:
        if self.isThis():
            if self.propertyName == b"geometry":
                value = self.targetObject().rect()
            elif self.propertyName == b"size":
                value = self.targetObject().size()
            elif self.propertyName == b"pos":
                value = self.targetObject().pos()
            elif self.propertyName == b"windowOpacity":
                value = self.targetObject().windowOpacity()
        super().setStartValue(value)


# 普通控件的特殊效果动画 - 阴影动画
class ShadowAnimation(PropertyAnimation):
    def __init__(self,parent:QObject,ani_data:dict,ani_obj_mode="control"):
        super().__init__(parent,ani_data,ani_obj_mode)

        if self.isEffect() is False:
            raise Exception("Special animation is not enabled,isEffect is False!")
        if self.targetObj.graphicsEffect() is None:
            self.__shadow = QGraphicsDropShadowEffect()
            self.targetObj.setGraphicsEffect(self.__shadow)
            self.__shadow.setOffset(0, 0)
        else:
            self.__shadow = self.targetObj.graphicsEffect()
        self.setPropertyName(self.propertyName)
        self.createAni()

    def setStartValue(self, value) -> None:
        value = value.value()
        if isinstance(value,int):
            self.__shadow.setBlurRadius(value)
            super().setStartValue(value)
        elif isinstance(value,QColor):
            self.__shadow.setColor(value)
            super().setStartValue(value)

    def updateCurrentValue(self, value: typing.Any) -> None:
        # print(value)
        if isinstance(value,int):
            print(value)
            self.__shadow.setBlurRadius(value)
        if isinstance(value,QColor):
            self.__shadow.setColor(value)


# 普通绘图 - 单值动画
class AnimationDrawValue(PropertyAnimation):
    def __init__(self,parent:QObject,ani_data:dict,ani_obj_mode="control"):
        super().__init__(parent,ani_data,ani_obj_mode)

        self.setPropertyName(b"value")
        self.createAni()

    def setStartValue(self, value) -> None:
        if isinstance(value,int):
            super().setStartValue(value)
        elif isinstance(value,AniNumber):
            super().setStartValue(value.value())

    def updateState(self, newState, oldState) -> None:
        super().updateState(newState, oldState)

    def updateCurrentValue(self, value) -> None:
        if self.drawSv():
            self.drawSv().setNumber(value)
            self.updateDraw()


# 返回动画类型
CreateAni = [AnimationControl,QSSPropertyAnimation,AnimationDrawValue]


'''
    动画工厂
'''
class AnimationFactory:
    # 动画对象模式
    Control = "control"  # 动画作用在普通控件上面
    Draw = "draw"        # 动画作用在绘制的图形上面

    def __init__(self,parent,ani_data:dict,ani_obj_mode):
        self.__ani_data = ani_data
        self.__parent = parent
        self.__ani_obj_mode = ani_obj_mode

    def parent(self):
        return self.__parent

    def aniData(self) -> dict:
        return self.__ani_data

    def propertyName(self)->str:
        return self.__ani_data['propertyName']

    def aniObjMode(self):
        return self.__ani_obj_mode

    def argc(self) -> tuple:
        return self.parent(),self.aniData(),self.aniObjMode()

    # 返回动画的实例
    def createAni(self) -> CreateAni:
        if self.aniObjMode() == AnimationFactory.Control:
            if self.propertyName() in [b"geometry",b"size",b"pos",b"windowOpacity"]:
                ani = AnimationControl(*self.argc())
            elif self.propertyName() == b"shadow":
                ani = ShadowAnimation(*self.argc())
            elif self.propertyName() and self.aniData().get("selector",None):  # 处理所有qss属性
                ani = QSSPropertyAnimation(*self.argc())
            else:
                raise Exception("There is no animation property,{}!".format(self.propertyName()))
        elif self.aniObjMode() == AnimationFactory.Draw:
            if self.propertyName() == b"value":
                ani = AnimationDrawValue(*self.argc())
            else:
                raise Exception("There is no animation property!")
        else:
            raise Exception("This animation mode is not available!")

        return ani


from PyQtGuiLib.header import (
    QWidget,
    QPushButton,
    QButtonGroup,
    QHBoxLayout,
    QStackedWidget,
    QLayout,
    QLabel,
    QVBoxLayout,
    qt,
    QSize,
    QCursor,
    Signal,
    QSizePolicy
)
from typing import List


class PageCuttingButtonGroup(QWidget):
    currentPageChanged = Signal(int)

    def __init__(self, totalPages, buttonCount, *args, **kwargs):
        super(PageCuttingButtonGroup, self).__init__(*args, **kwargs)
        self._totalPages = totalPages
        if self._totalPages < 2:
            return

        self._buttonCount = max(5, buttonCount)
        self._currentPageIndex = -1
        self.__initializeWidget()
        self.setCurrentPageIndex(0)

        self.__personalInformation = {
            "author":"PyQt5学习爱好群-讨厌自己",
            "contribution time":"2023.4.25"
        }

        self.defaultStyle()

    def author(self) -> str:
        return self.__personalInformation

    def bingStackedWidget(self,st:QStackedWidget):
        self.currentPageChanged.connect(st.setCurrentIndex)

    def setSpacing(self, a0: int) -> None:
        if self._totalPages < 2:
            return

        self._hBox.setSpacing(a0)

    def setCurrentPageButtonFixedSize(self, size: QSize) -> None:
        if self._totalPages < 2:
            return

        for i, button in enumerate(self._group.buttons()):
            button.setFixedSize(size)

    def setPreviousNextPageButtonFixedSize(self, size: QSize) -> None:
        if self._totalPages < 2:
            return

        self._previousPageButton.setFixedSize(size)
        self._nextPageButton.setFixedSize(size)

    def currentPageIndex(self) -> int:
        return self._currentPageIndex

    def setCurrentPageIndex(self, index: int) -> None:
        if self._totalPages < 2 or self._totalPages <= index or index < 0:
            return
        self.__clickedOnTheJudgmentButton(currentPageIndex=index)

    def __previousOrNextPageButtonClicked(self, text):
        if text == "下一页" and self._currentPageIndex < self._totalPages - 1:
            currentPageIndex = self._currentPageIndex + 1
        elif text == "上一页" and self._currentPageIndex > 0:
            currentPageIndex = self._currentPageIndex - 1
        else:
            return
        self.__clickedOnTheJudgmentButton(currentPageIndex=currentPageIndex)

    @staticmethod
    def assignValues(middleValue, lst: List[QPushButton], pos: list):
        length = len(lst)
        middleIndex = length // 2
        lst[middleIndex].setText(str(middleValue))
        for i in range(1, middleIndex):
            lst[i].setText("...") if i in pos else lst[i].setText(str(middleValue - (middleIndex - i)))
        for i in range(middleIndex + 1, length - 1):
            lst[i].setText("...") if i in pos else lst[i].setText(str(middleValue + (i - middleIndex)))

        return lst

    def __clickedOnTheJudgmentButton(self, currentPageIndex: int):
        if currentPageIndex == self._currentPageIndex:
            return

        centre = self._buttonCount // 2 + 1
        if self._totalPages > self._buttonCount:
            if currentPageIndex < centre:
                self.assignValues(middleValue=centre, lst=self._group.buttons(), pos=[self._buttonCount - 2])
            elif centre <= currentPageIndex < self._totalPages - centre:
                self.assignValues(middleValue=currentPageIndex + 1, lst=self._group.buttons(),
                                  pos=[1, self._buttonCount - 2])
            elif currentPageIndex >= self._totalPages - centre:
                self.assignValues(middleValue=self._totalPages - centre + 1, lst=self._group.buttons(), pos=[1])
        self._currentPageIndex = currentPageIndex
        self.currentPageChanged.emit(currentPageIndex)

        for i, button in enumerate(self._group.buttons()):
            if button.text() == str(currentPageIndex + 1):
                button.setCheckable(True)
                button.setChecked(True)
            else:
                button.setCheckable(False)
                button.update()
            button.setEnabled(False) if button.text() == "..." else button.setEnabled(True)

        self.__setButtonEnabled(self._previousPageButton, False) if self._currentPageIndex == 0 \
            else self.__setButtonEnabled(self._previousPageButton, True)
        self.__setButtonEnabled(self._nextPageButton, False) if self._currentPageIndex == self._totalPages - 1 \
            else self.__setButtonEnabled(self._nextPageButton, True)

    def __initializeWidget(self):
        self._group = QButtonGroup(parent=self)
        self._group.buttonClicked.connect(
            lambda button: self.__clickedOnTheJudgmentButton(currentPageIndex=int(button.text()) - 1))
        self._hBox = QHBoxLayout(self)
        self._hBox.setSizeConstraint(QLayout.SizeConstraint.SetFixedSize)
        self._previousPageButton = self.__registerButton("上一页", False)
        self._previousPageButton.clicked.connect(lambda: self.__previousOrNextPageButtonClicked(text="上一页"))
        self._nextPageButton = self.__registerButton("下一页", False)
        self._nextPageButton.clicked.connect(lambda: self.__previousOrNextPageButtonClicked(text="下一页"))
        self._hBox.addWidget(self._previousPageButton)
        if self._totalPages <= self._buttonCount:
            for i in range(self._totalPages):
                button = self.__registerButton("{}".format(i+1), True)
                self._hBox.addWidget(button)
                self._group.addButton(button, id=i)
        else:
            for i in range(self._buttonCount):
                if i == self._buttonCount - 1:
                    button = self.__registerButton(str(self._totalPages), True)
                else:
                    button = self.__registerButton(str(i + 1), True)
                self._hBox.addWidget(button)
                self._group.addButton(button, id=i)
        self._hBox.addWidget(self._nextPageButton)

    def __setButtonEnabled(self, button: QPushButton, enabled: bool):
        if enabled:
            button.setCursor(QCursor(qt.PointingHandCursor))
            button.setStyleSheet("")
        else:
            button.setCursor(QCursor(qt.ForbiddenCursor))
            button.setStyleSheet("""
                QPushButton{
                    border: 1px solid rgba(241, 242, 243, 255);
                    color: rgb(203, 206, 210);
                }
                QPushButton:hover{
                    background-color:none;
                }
            """)

        return button

    def __registerButton(self, text: str, checkAble: bool):
        button = QPushButton(text, parent=self)
        button.setCheckable(True) if checkAble else None
        button.setCursor(QCursor(qt.PointingHandCursor))
        button.setSizePolicy(QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed))

        return button

    def defaultStyle(self):
        self.setStyleSheet('''
            QWidget{
                background-color: transparent;
            }
            QPushButton{
                font: 14px "黑体";
                color: rgba(84, 87, 87, 255);
                border:1px solid rgba(227, 229, 231, 255);
                border-radius:6px;
            }
            QPushButton:hover{
                background-color: rgba(227, 229, 231, 255);
            }
            QPushButton:checked{
                background-color: rgba(255, 102, 153, 255);
                color: rgba(255, 255, 255, 255);
                border:1px solid rgba(255, 102, 153, 255);
            }
            QPushButton:disabled{
                border:none;
                background-color:transparent;
            }
        ''')


from PyQtGuiLib.styles.skinABC import SkinABC
from PyQtGuiLib.styles.buttons.buttonStyle import ButtonStyle
from PyQtGuiLib.styles.styleAnalysis import QssStyleAnalysis
# from PyQtGuiLib.styles.linker.styleLinker import StyleLinker
from PyQtGuiLib.styles.superPainter.superPainter import SuperPainter
from PyQtGuiLib.styles.qssFile.drak.qssDrak import QSSDrak


# -*- coding:utf-8 -*-
# @time:2023/3/69:33
# @author:LX
# @file:toolBox.py
# @software:PyCharm

from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QToolBox,
    QWidget,
    QIcon,
    QListWidget,
    QListWidgetItem,
    QListView,
    QSize,
    QFont
)

class ToolBox(QToolBox):
    def __init__(self,*args,**kwargs):
        super().__init__()

        self.resize(300,300)
        self.setWindowTitle("ToolBox")
        self.layout().setSpacing(0)


        # item默认的图标大小
        self.default_item_icon_size = QSize(32, 32)

        self.addTextNode({
            "title":"page1",
            "icon":r"D:\code\PyQtGuiLib\tests\temp_image\python1.png",
            "item":{
                "text": "测试",
                "icon":r"D:\code\PyQtGuiLib\tests\temp_image\python1.png",
                "size":QSize(50,50)
            }
        })

        self.addTextNode({
            "title":"page1",
            "icon":r"D:\code\PyQtGuiLib\tests\temp_image\python1.png",
            "item":{
                "text": "测试",
                "icon":r"D:\code\PyQtGuiLib\tests\temp_image\python1.png",
                "size":QSize(50,50)
            }
        })


    def addTextNode(self,data:dict):
        '''
            {
                "title":""
                "icon":None,
                "item":{
                    "icon":None,
                    "text":"",
                    "size":QSize() or None
                }
            }

        :param data:
        :return:
        '''
        title_text = data.get("title")
        title_icon = data.get("icon")
        item_icon = data.get("item").get("icon")
        item_text = data.get("item").get("text")
        item_size = data.get("item").get("size")
        print(item_size)

        node = QListWidget()
        if item_size:
            node.setIconSize(item_size)
        else:
            node.setIconSize(self.default_item_icon_size)
        item_node = QListWidgetItem()
        item_node.setText(item_text)
        if item_icon:
            item_node.setIcon(QIcon(item_icon))
        node.addItem(item_node)

        if title_icon:
            self.addItem(node,QIcon(title_icon),title_text)
        else:
            self.addItem(node,title_text)



    # def addItem(self, item: QWidget, text: str,icon:QIcon=None) -> int:
    #
    #     if icon:
    #         super(ToolBox, self).addItem(item,icon,text)
    #     else:
    #         super(ToolBox, self).addItem(item, text)


if __name__ == '__main__':
    app = QApplication(sys.argv)

    win = ToolBox()
    win.show()

    sys.exit(app.exec_())


from PyQtGuiLib.header import (
    QWidget,
    QBrush,
    QPainter,
    QRect,
    QColor,
    QPaintEvent,
    QMouseEvent,
    qt,
    Signal
)

'''
    开关按钮
'''
class SwitchButton(QWidget):
    clicked = Signal(bool)

    # --- 形状
    Shape_Circle = "circle"
    Shape_Square = "square"

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(130,60)

        self.setAttribute(qt.WA_TranslucentBackground)
        self.setWindowFlags(qt.FramelessWindowHint | qt.Widget)

        # 开关标记
        self.is_switch = True

        self.ellipse_x = 1

        self.startTimer(15)
        self.start_color = QColor(0,255,0)
        self.end_color = QColor(255,0,0)
        self.cu_color = self.start_color

        # 背景颜色,球的颜色
        self.bg_color = {"false":QColor(177, 177, 177),"true":QColor(0, 255, 127)}
        self.ball_color = {"false":QColor(234, 234, 234),"true":QColor(0, 255, 127)}

        # 形状
        self.shape = SwitchButton.Shape_Circle

    # 设置形状
    def setShape(self,shape:str):
        self.shape = shape

    # 设置背景颜色
    def setBgColor(self,color_dict:dict):
        '''
            {
                "false":QColor,
                "true":QColor
            }
        '''
        self.bg_color = color_dict

    # 设置运行球的颜色
    def setBallColor(self,color_dict:dict):
        '''
            {
                "false":QColor,
                "true":QColor
            }
        '''
        self.ball_color = color_dict

    # 返回当前的状态
    def state(self) -> bool:
        return not self.is_switch

    # 设置默认状态
    def setDefaultState(self,state:bool):
        self.is_switch = not state
        # self.update()

    def mousePressEvent(self, e: QMouseEvent) -> None:
        self.is_switch = False if self.is_switch else True
        self.clicked.emit(self.state())
        super().mousePressEvent(e)

    def drawSquareSW(self,painter:QPainter):
        # 画背景颜色
        bg = QBrush(self.bg_color["false"])
        painter.setBrush(bg)
        painter.setPen(qt.NoPen)
        rect_ = QRect(1, 1, self.width() - 3, self.height() - 2)
        painter.drawRoundedRect(rect_, 3, 3)

        # 画球的颜色
        bru = QBrush(self.ball_color["false"])
        painter.setBrush(bru)
        if self.ellipse_x < 0:
            self.ellipse_x = 1
        painter.drawRoundedRect(self.ellipse_x+2, 3, self.height()-3, self.height() - 2-4, 3, 3)
        # painter.drawEllipse(self.ellipse_x, 1, self.height(), self.height() - 2)

        # ----
        if self.is_switch is False:
            bg = QBrush(self.bg_color["true"])
            painter.setBrush(bg)
            painter.drawRoundedRect(rect_, 3, 3)
            # 画球的颜色
            bru = QBrush(self.ball_color["false"])
            painter.setBrush(bru)
            # painter.drawEllipse(self.ellipse_x, 1, self.height(), self.height() - 2)
            painter.drawRoundedRect(self.ellipse_x+2, 3, self.height()-3, self.height() - 2-4, 3, 3)

    # 绘制开关按钮 --
    def drawSwButton(self,painter:QPainter):
        # 画背景颜色
        bg = QBrush(self.bg_color["false"])
        painter.setBrush(bg)
        painter.setPen(qt.NoPen)
        rect_ = QRect(1, 1, self.width() - 3, self.height() - 2)
        painter.drawRoundedRect(rect_, self.height() // 2, self.height() // 2)

        # 画球的颜色
        bru = QBrush(self.ball_color["false"])
        painter.setBrush(bru)
        if self.ellipse_x < 0:
            self.ellipse_x = 1
        painter.drawEllipse(self.ellipse_x, 1, self.height(), self.height() - 2)

        # ----
        if self.is_switch is False:
            bg = QBrush(self.bg_color["true"])
            painter.setBrush(bg)
            painter.drawRoundedRect(rect_, self.height() // 2, self.height() // 2)
            # 画球的颜色
            bru = QBrush(self.ball_color["false"])
            painter.setBrush(bru)
            painter.drawEllipse(self.ellipse_x, 1, self.height(), self.height() - 2)

    def paintEvent(self, e: QPaintEvent) -> None:
        painter = QPainter(self)
        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform | qt.TextAntialiasing)

        # 绘制开关按钮
        if self.shape == SwitchButton.Shape_Square:
            self.drawSquareSW(painter)
        else:
            self.drawSwButton(painter)

        painter.end()

    def timerEvent(self,e) -> None:
        if not self.is_switch and self.ellipse_x < self.width()-self.height()-3:
            self.ellipse_x += (self.width()-self.height()-3)//20
            self.repaint()
        elif self.is_switch and self.ellipse_x >= 2:
            self.ellipse_x -= (self.width()-self.height()-3)//20
            self.repaint()



# -*- coding:utf-8 -*-
# @time:2022/12/1211:40
# @author:LX
# @file:__init__.py.py
# @software:PyCharm
from PyQtGuiLib.animation.animation import (
    Animation,
    ParallelAnimationGroup,
    SequentialAnimationGroup
)
